import Security
from PyObjCTools.TestSupport import TestCase
import objc


class TestSecImportExport(TestCase):
    def test_constants(self):
        self.assertEqual(Security.kSecFormatUnknown, 0)
        self.assertEqual(Security.kSecFormatOpenSSL, 1)
        self.assertEqual(Security.kSecFormatSSH, 2)
        self.assertEqual(Security.kSecFormatBSAFE, 3)
        self.assertEqual(Security.kSecFormatRawKey, 4)
        self.assertEqual(Security.kSecFormatWrappedPKCS8, 5)
        self.assertEqual(Security.kSecFormatWrappedOpenSSL, 6)
        self.assertEqual(Security.kSecFormatWrappedSSH, 7)
        self.assertEqual(Security.kSecFormatWrappedLSH, 8)
        self.assertEqual(Security.kSecFormatX509Cert, 9)
        self.assertEqual(Security.kSecFormatPEMSequence, 10)
        self.assertEqual(Security.kSecFormatPKCS7, 11)
        self.assertEqual(Security.kSecFormatPKCS12, 12)
        self.assertEqual(Security.kSecFormatNetscapeCertSequence, 13)
        self.assertEqual(Security.kSecFormatSSHv2, 14)

        self.assertEqual(Security.kSecItemTypeUnknown, 0)
        self.assertEqual(Security.kSecItemTypePrivateKey, 1)
        self.assertEqual(Security.kSecItemTypePublicKey, 2)
        self.assertEqual(Security.kSecItemTypeSessionKey, 3)
        self.assertEqual(Security.kSecItemTypeCertificate, 4)
        self.assertEqual(Security.kSecItemTypeAggregate, 5)

        self.assertEqual(Security.kSecItemPemArmour, 0x00000001)

        self.assertEqual(Security.kSecKeyImportOnlyOne, 0x00000001)
        self.assertEqual(Security.kSecKeySecurePassphrase, 0x00000002)
        self.assertEqual(Security.kSecKeyNoAccessControl, 0x00000004)

        self.assertEqual(Security.SEC_KEY_IMPORT_EXPORT_PARAMS_VERSION, 0)

    def test_structs(self):
        v = Security.SecItemImportExportKeyParameters()
        self.assertEqual(v.version, 0)
        self.assertEqual(v.flags, 0)
        self.assertEqual(v.passphrase, None)
        self.assertEqual(v.alertTitle, None)
        self.assertEqual(v.alertPrompt, None)
        self.assertEqual(v.accessRef, None)
        self.assertEqual(v.keyUsage, None)
        self.assertEqual(v.keyAttributes, None)

        self.assertIsInstance(Security.kSecImportExportPassphrase, str)
        self.assertIsInstance(Security.kSecImportExportKeychain, str)
        self.assertIsInstance(Security.kSecImportExportAccess, str)

        self.assertIsInstance(Security.kSecImportItemLabel, str)
        self.assertIsInstance(Security.kSecImportItemKeyID, str)
        self.assertIsInstance(Security.kSecImportItemTrust, str)
        self.assertIsInstance(Security.kSecImportItemCertChain, str)
        self.assertIsInstance(Security.kSecImportItemIdentity, str)

    def test_functions(self):
        self.assertFalse(hasattr(Security, "SecKeychainItemExport"))

        self.assertResultHasType(Security.SecItemExport, objc._C_INT)
        self.assertArgHasType(Security.SecItemExport, 0, objc._C_ID)
        self.assertArgHasType(Security.SecItemExport, 1, objc._C_UINT)
        self.assertArgHasType(Security.SecItemExport, 2, objc._C_UINT)
        self.assertArgHasType(
            Security.SecItemExport,
            3,
            objc._C_IN
            + objc._C_PTR
            + Security.SecItemImportExportKeyParameters.__typestr__,
        )
        self.assertArgHasType(
            Security.SecItemExport, 4, objc._C_OUT + objc._C_PTR + objc._C_ID
        )
        self.assertArgIsCFRetained(Security.SecItemExport, 4)

        self.assertFalse(hasattr(Security, "SecKeychainItemImport"))

        self.assertResultHasType(Security.SecItemImport, objc._C_INT)
        self.assertArgHasType(Security.SecItemImport, 0, objc._C_ID)
        self.assertArgHasType(Security.SecItemImport, 1, objc._C_ID)
        self.assertArgHasType(
            Security.SecItemImport, 2, objc._C_INOUT + objc._C_PTR + objc._C_UINT
        )
        self.assertArgHasType(
            Security.SecItemImport, 3, objc._C_INOUT + objc._C_PTR + objc._C_UINT
        )
        self.assertArgHasType(Security.SecItemImport, 4, objc._C_UINT)
        self.assertArgHasType(
            Security.SecItemImport,
            5,
            objc._C_IN
            + objc._C_PTR
            + Security.SecItemImportExportKeyParameters.__typestr__,
        )
        self.assertArgHasType(Security.SecItemImport, 6, objc._C_ID)
        self.assertArgHasType(
            Security.SecItemImport, 7, objc._C_OUT + objc._C_PTR + objc._C_ID
        )
        self.assertArgIsCFRetained(Security.SecItemImport, 7)

        self.assertResultHasType(Security.SecPKCS12Import, objc._C_INT)
        self.assertArgHasType(Security.SecPKCS12Import, 0, objc._C_ID)
        self.assertArgHasType(Security.SecPKCS12Import, 1, objc._C_ID)
        self.assertArgHasType(
            Security.SecPKCS12Import, 2, objc._C_OUT + objc._C_PTR + objc._C_ID
        )
        self.assertArgIsCFRetained(Security.SecPKCS12Import, 2)



import Security
from PyObjCTools.TestSupport import TestCase


class Testoidscrl(TestCase):
    def test_unsuppported(self):
        self.assertFalse(hasattr(Security, "INTEL_X509V2_CRL_R08"))
        self.assertFalse(hasattr(Security, "INTEL_X509V2_CRL_R08_LENGTH"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V2CRLSignedCrlStruct"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V2CRLSignedCrlCStruct"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V2CRLTbsCertListStruct"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V2CRLTbsCertListCStruct"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V2CRLVersion"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V1CRLIssuerStruct"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V1CRLIssuerNameCStruct"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V1CRLIssuerNameLDAP"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V1CRLThisUpdate"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V1CRLNextUpdate"))
        self.assertFalse(
            hasattr(Security, "CSSMOID_X509V1CRLRevokedCertificatesStruct")
        )
        self.assertFalse(
            hasattr(Security, "CSSMOID_X509V1CRLRevokedCertificatesCStruct")
        )
        self.assertFalse(
            hasattr(Security, "CSSMOID_X509V1CRLNumberOfRevokedCertEntries")
        )
        self.assertFalse(hasattr(Security, "CSSMOID_X509V1CRLRevokedEntryStruct"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V1CRLRevokedEntryCStruct"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V1CRLRevokedEntrySerialNumber"))
        self.assertFalse(
            hasattr(Security, "CSSMOID_X509V1CRLRevokedEntryRevocationDate")
        )
        self.assertFalse(
            hasattr(Security, "CSSMOID_X509V2CRLRevokedEntryAllExtensionsStruct")
        )
        self.assertFalse(
            hasattr(Security, "CSSMOID_X509V2CRLRevokedEntryAllExtensionsCStruct")
        )
        self.assertFalse(
            hasattr(Security, "CSSMOID_X509V2CRLRevokedEntryNumberOfExtensions")
        )
        self.assertFalse(
            hasattr(Security, "CSSMOID_X509V2CRLRevokedEntrySingleExtensionStruct")
        )
        self.assertFalse(
            hasattr(Security, "CSSMOID_X509V2CRLRevokedEntrySingleExtensionCStruct")
        )
        self.assertFalse(hasattr(Security, "CSSMOID_X509V2CRLRevokedEntryExtensionId"))
        self.assertFalse(
            hasattr(Security, "CSSMOID_X509V2CRLRevokedEntryExtensionCritical")
        )
        self.assertFalse(
            hasattr(Security, "CSSMOID_X509V2CRLRevokedEntryExtensionType")
        )
        self.assertFalse(
            hasattr(Security, "CSSMOID_X509V2CRLRevokedEntryExtensionValue")
        )
        self.assertFalse(hasattr(Security, "CSSMOID_X509V2CRLAllExtensionsStruct"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V2CRLAllExtensionsCStruct"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V2CRLNumberOfExtensions"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V2CRLSingleExtensionStruct"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V2CRLSingleExtensionCStruct"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V2CRLExtensionId"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V2CRLExtensionCritical"))
        self.assertFalse(hasattr(Security, "CSSMOID_X509V2CRLExtensionType"))
        self.assertFalse(hasattr(Security, "CSSMOID_PKIX_OCSP"))
        self.assertFalse(hasattr(Security, "CSSMOID_PKIX_OCSP_BASIC"))
        self.assertFalse(hasattr(Security, "CSSMOID_PKIX_OCSP_NONCE"))
        self.assertFalse(hasattr(Security, "CSSMOID_PKIX_OCSP_CRL"))
        self.assertFalse(hasattr(Security, "CSSMOID_PKIX_OCSP_RESPONSE"))
        self.assertFalse(hasattr(Security, "CSSMOID_PKIX_OCSP_NOCHECK"))
        self.assertFalse(hasattr(Security, "CSSMOID_PKIX_OCSP_ARCHIVE_CUTOFF"))
        self.assertFalse(hasattr(Security, "CSSMOID_PKIX_OCSP_SERVICE_LOCATOR"))



import Security
from PyObjCTools.TestSupport import TestCase
import objc


class TestSecSignVerifyTransform(TestCase):
    def test_constants(self):
        self.assertIsInstance(Security.kSecKeyAttributeName, str)
        self.assertIsInstance(Security.kSecSignatureAttributeName, str)
        self.assertIsInstance(Security.kSecInputIsAttributeName, str)
        self.assertIsInstance(Security.kSecInputIsPlainText, str)
        self.assertIsInstance(Security.kSecInputIsDigest, str)
        self.assertIsInstance(Security.kSecInputIsRaw, str)

    def test_functions(self):
        self.assertResultHasType(Security.SecSignTransformCreate, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecSignTransformCreate)
        self.assertArgHasType(Security.SecSignTransformCreate, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecSignTransformCreate, 1, objc._C_OUT + objc._C_PTR + objc._C_ID
        )

        self.assertResultHasType(Security.SecVerifyTransformCreate, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecVerifyTransformCreate)
        self.assertArgHasType(Security.SecVerifyTransformCreate, 0, objc._C_ID)
        self.assertArgHasType(Security.SecVerifyTransformCreate, 1, objc._C_ID)
        self.assertArgHasType(
            Security.SecVerifyTransformCreate, 2, objc._C_OUT + objc._C_PTR + objc._C_ID
        )



import Security
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestCSCommon(TestCase):
    def test_types(self):
        self.assertIsCFType(Security.SecCodeRef)
        self.assertIsCFType(Security.SecRequirementRef)

    def test_constants(self):
        self.assertEqual(Security.errSecCSUnimplemented, -67072)
        self.assertEqual(Security.errSecCSInvalidObjectRef, -67071)
        self.assertEqual(Security.errSecCSInvalidFlags, -67070)
        self.assertEqual(Security.errSecCSObjectRequired, -67069)
        self.assertEqual(Security.errSecCSStaticCodeNotFound, -67068)
        self.assertEqual(Security.errSecCSUnsupportedGuestAttributes, -67067)
        self.assertEqual(Security.errSecCSInvalidAttributeValues, -67066)
        self.assertEqual(Security.errSecCSNoSuchCode, -67065)
        self.assertEqual(Security.errSecCSMultipleGuests, -67064)
        self.assertEqual(Security.errSecCSGuestInvalid, -67063)
        self.assertEqual(Security.errSecCSUnsigned, -67062)
        self.assertEqual(Security.errSecCSSignatureFailed, -67061)
        self.assertEqual(Security.errSecCSSignatureNotVerifiable, -67060)
        self.assertEqual(Security.errSecCSSignatureUnsupported, -67059)
        self.assertEqual(Security.errSecCSBadDictionaryFormat, -67058)
        self.assertEqual(Security.errSecCSResourcesNotSealed, -67057)
        self.assertEqual(Security.errSecCSResourcesNotFound, -67056)
        self.assertEqual(Security.errSecCSResourcesInvalid, -67055)
        self.assertEqual(Security.errSecCSBadResource, -67054)
        self.assertEqual(Security.errSecCSResourceRulesInvalid, -67053)
        self.assertEqual(Security.errSecCSReqInvalid, -67052)
        self.assertEqual(Security.errSecCSReqUnsupported, -67051)
        self.assertEqual(Security.errSecCSReqFailed, -67050)
        self.assertEqual(Security.errSecCSBadObjectFormat, -67049)
        self.assertEqual(Security.errSecCSInternalError, -67048)
        self.assertEqual(Security.errSecCSHostReject, -67047)
        self.assertEqual(Security.errSecCSNotAHost, -67046)
        self.assertEqual(Security.errSecCSSignatureInvalid, -67045)
        self.assertEqual(Security.errSecCSHostProtocolRelativePath, -67044)
        self.assertEqual(Security.errSecCSHostProtocolContradiction, -67043)
        self.assertEqual(Security.errSecCSHostProtocolDedicationError, -67042)
        self.assertEqual(Security.errSecCSHostProtocolNotProxy, -67041)
        self.assertEqual(Security.errSecCSHostProtocolStateError, -67040)
        self.assertEqual(Security.errSecCSHostProtocolUnrelated, -67039)
        self.assertEqual(Security.errSecCSNotSupported, -67037)
        self.assertEqual(Security.errSecCSCMSTooLarge, -67036)
        self.assertEqual(Security.errSecCSHostProtocolInvalidHash, -67035)
        self.assertEqual(Security.errSecCSStaticCodeChanged, -67034)
        self.assertEqual(Security.errSecCSDBDenied, -67033)
        self.assertEqual(Security.errSecCSDBAccess, -67032)
        self.assertEqual(Security.errSecCSSigDBDenied, Security.errSecCSDBDenied)
        self.assertEqual(Security.errSecCSSigDBAccess, Security.errSecCSDBAccess)
        self.assertEqual(Security.errSecCSHostProtocolInvalidAttribute, -67031)
        self.assertEqual(Security.errSecCSInfoPlistFailed, -67030)
        self.assertEqual(Security.errSecCSNoMainExecutable, -67029)
        self.assertEqual(Security.errSecCSBadBundleFormat, -67028)
        self.assertEqual(Security.errSecCSNoMatches, -67027)
        self.assertEqual(Security.errSecCSFileHardQuarantined, -67026)
        self.assertEqual(Security.errSecCSOutdated, -67025)
        self.assertEqual(Security.errSecCSDbCorrupt, -67024)
        self.assertEqual(Security.errSecCSResourceDirectoryFailed, -67023)
        self.assertEqual(Security.errSecCSUnsignedNestedCode, -67022)
        self.assertEqual(Security.errSecCSBadNestedCode, -67021)
        self.assertEqual(Security.errSecCSBadCallbackValue, -67020)
        self.assertEqual(Security.errSecCSHelperFailed, -67019)
        self.assertEqual(Security.errSecCSVetoed, -67018)
        self.assertEqual(Security.errSecCSBadLVArch, -67017)
        self.assertEqual(Security.errSecCSResourceNotSupported, -67016)
        self.assertEqual(Security.errSecCSRegularFile, -67015)
        self.assertEqual(Security.errSecCSUnsealedAppRoot, -67014)
        self.assertEqual(Security.errSecCSWeakResourceRules, -67013)
        self.assertEqual(Security.errSecCSDSStoreSymlink, -67012)
        self.assertEqual(Security.errSecCSAmbiguousBundleFormat, -67011)
        self.assertEqual(Security.errSecCSBadMainExecutable, -67010)
        self.assertEqual(Security.errSecCSBadFrameworkVersion, -67009)
        self.assertEqual(Security.errSecCSUnsealedFrameworkRoot, -67008)
        self.assertEqual(Security.errSecCSWeakResourceEnvelope, -67007)
        self.assertEqual(Security.errSecCSCancelled, -67006)
        self.assertEqual(Security.errSecCSInvalidPlatform, -67005)
        self.assertEqual(Security.errSecCSTooBig, -67004)
        self.assertEqual(Security.errSecCSInvalidSymlink, -67003)
        self.assertEqual(Security.errSecCSNotAppLike, -67002)
        self.assertEqual(Security.errSecCSBadDiskImageFormat, -67001)
        self.assertEqual(Security.errSecCSUnsupportedDigestAlgorithm, -67000)
        self.assertEqual(Security.errSecCSInvalidAssociatedFileData, -66999)
        self.assertEqual(Security.errSecCSInvalidTeamIdentifier, -66998)
        self.assertEqual(Security.errSecCSBadTeamIdentifier, -66997)
        self.assertEqual(Security.errSecCSSignatureUntrusted, -66996)
        self.assertEqual(Security.errSecMultipleExecSegments, -66995)
        self.assertEqual(Security.errSecCSInvalidEntitlements, -66994)
        self.assertEqual(Security.errSecCSInvalidRuntimeVersion, -66993)
        self.assertEqual(Security.errSecCSRevokedNotarization, -66992)

        self.assertIsInstance(Security.kSecCFErrorArchitecture, str)
        self.assertIsInstance(Security.kSecCFErrorPattern, str)
        self.assertIsInstance(Security.kSecCFErrorResourceSeal, str)
        self.assertIsInstance(Security.kSecCFErrorResourceAdded, str)
        self.assertIsInstance(Security.kSecCFErrorResourceAltered, str)
        self.assertIsInstance(Security.kSecCFErrorResourceMissing, str)
        self.assertIsInstance(Security.kSecCFErrorInfoPlist, str)
        self.assertIsInstance(Security.kSecCFErrorGuestAttributes, str)
        self.assertIsInstance(Security.kSecCFErrorRequirementSyntax, str)
        self.assertIsInstance(Security.kSecCFErrorPath, str)

        self.assertEqual(Security.kSecNoGuest, 0)

        self.assertEqual(Security.kSecCSDefaultFlags, 0)
        self.assertEqual(Security.kSecCSConsiderExpiration, 1 << 31)
        self.assertEqual(Security.kSecCSEnforceRevocationChecks, 1 << 30)
        self.assertEqual(Security.kSecCSNoNetworkAccess, 1 << 29)
        self.assertEqual(Security.kSecCSReportProgress, 1 << 28)
        self.assertEqual(Security.kSecCSCheckTrustedAnchors, 1 << 27)
        self.assertEqual(Security.kSecCSQuickCheck, 1 << 26)
        self.assertEqual(Security.kSecCSApplyEmbeddedPolicy, 1 << 25)

        self.assertEqual(Security.kSecCodeSignatureHost, 0x0001)
        self.assertEqual(Security.kSecCodeSignatureAdhoc, 0x0002)
        self.assertEqual(Security.kSecCodeSignatureForceHard, 0x0100)
        self.assertEqual(Security.kSecCodeSignatureForceKill, 0x0200)
        self.assertEqual(Security.kSecCodeSignatureForceExpiration, 0x0400)
        self.assertEqual(Security.kSecCodeSignatureRestrict, 0x0800)
        self.assertEqual(Security.kSecCodeSignatureEnforcement, 0x1000)
        self.assertEqual(Security.kSecCodeSignatureLibraryValidation, 0x2000)
        self.assertEqual(Security.kSecCodeSignatureRuntime, 0x10000)
        self.assertEqual(Security.kSecCodeSignatureLinkerSigned, 0x20000)

        self.assertEqual(Security.kSecCodeStatusValid, 0x0001)
        self.assertEqual(Security.kSecCodeStatusHard, 0x0100)
        self.assertEqual(Security.kSecCodeStatusKill, 0x0200)
        self.assertEqual(Security.kSecCodeStatusDebugged, 0x10000000)
        self.assertEqual(Security.kSecCodeStatusPlatform, 0x04000000)

        self.assertEqual(Security.kSecHostRequirementType, 1)
        self.assertEqual(Security.kSecGuestRequirementType, 2)
        self.assertEqual(Security.kSecDesignatedRequirementType, 3)
        self.assertEqual(Security.kSecLibraryRequirementType, 4)
        self.assertEqual(Security.kSecPluginRequirementType, 5)
        self.assertEqual(Security.kSecInvalidRequirementType, 6)
        self.assertEqual(
            Security.kSecRequirementTypeCount, Security.kSecInvalidRequirementType
        )

        self.assertEqual(Security.kSecCodeSignatureNoHash, 0)
        self.assertEqual(Security.kSecCodeSignatureHashSHA1, 1)
        self.assertEqual(Security.kSecCodeSignatureHashSHA256, 2)
        self.assertEqual(Security.kSecCodeSignatureHashSHA256Truncated, 3)
        self.assertEqual(Security.kSecCodeSignatureHashSHA384, 4)
        self.assertEqual(Security.kSecCodeSignatureHashSHA512, 5)

    @min_os_level("10.12")
    def test_constants_10_12(self):
        self.assertIsInstance(Security.kSecCFErrorResourceSideband, str)



import Security
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestSecProtocolTypes(TestCase):
    def test_constants(self):
        self.assertEqual(Security.tls_protocol_version_TLSv10, 0x0301)
        self.assertEqual(Security.tls_protocol_version_TLSv11, 0x0302)
        self.assertEqual(Security.tls_protocol_version_TLSv12, 0x0303)
        self.assertEqual(Security.tls_protocol_version_TLSv13, 0x0304)
        self.assertEqual(Security.tls_protocol_version_DTLSv10, 0xFEFF)
        self.assertEqual(Security.tls_protocol_version_DTLSv12, 0xFEFD)

        self.assertEqual(Security.tls_ciphersuite_RSA_WITH_3DES_EDE_CBC_SHA, 0x000A)
        self.assertEqual(Security.tls_ciphersuite_RSA_WITH_AES_128_CBC_SHA, 0x002F)
        self.assertEqual(Security.tls_ciphersuite_RSA_WITH_AES_256_CBC_SHA, 0x0035)
        self.assertEqual(Security.tls_ciphersuite_RSA_WITH_AES_128_GCM_SHA256, 0x009C)
        self.assertEqual(Security.tls_ciphersuite_RSA_WITH_AES_256_GCM_SHA384, 0x009D)
        self.assertEqual(Security.tls_ciphersuite_RSA_WITH_AES_128_CBC_SHA256, 0x003C)
        self.assertEqual(Security.tls_ciphersuite_RSA_WITH_AES_256_CBC_SHA256, 0x003D)
        self.assertEqual(
            Security.tls_ciphersuite_ECDHE_ECDSA_WITH_3DES_EDE_CBC_SHA, 0xC008
        )
        self.assertEqual(
            Security.tls_ciphersuite_ECDHE_ECDSA_WITH_AES_128_CBC_SHA, 0xC009
        )
        self.assertEqual(
            Security.tls_ciphersuite_ECDHE_ECDSA_WITH_AES_256_CBC_SHA, 0xC00A
        )
        self.assertEqual(
            Security.tls_ciphersuite_ECDHE_RSA_WITH_3DES_EDE_CBC_SHA, 0xC012
        )
        self.assertEqual(
            Security.tls_ciphersuite_ECDHE_RSA_WITH_AES_128_CBC_SHA, 0xC013
        )
        self.assertEqual(
            Security.tls_ciphersuite_ECDHE_RSA_WITH_AES_256_CBC_SHA, 0xC014
        )
        self.assertEqual(
            Security.tls_ciphersuite_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256, 0xC023
        )
        self.assertEqual(
            Security.tls_ciphersuite_ECDHE_ECDSA_WITH_AES_256_CBC_SHA384, 0xC024
        )
        self.assertEqual(
            Security.tls_ciphersuite_ECDHE_RSA_WITH_AES_128_CBC_SHA256, 0xC027
        )
        self.assertEqual(
            Security.tls_ciphersuite_ECDHE_RSA_WITH_AES_256_CBC_SHA384, 0xC028
        )
        self.assertEqual(
            Security.tls_ciphersuite_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256, 0xC02B
        )
        self.assertEqual(
            Security.tls_ciphersuite_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384, 0xC02C
        )
        self.assertEqual(
            Security.tls_ciphersuite_ECDHE_RSA_WITH_AES_128_GCM_SHA256, 0xC02F
        )
        self.assertEqual(
            Security.tls_ciphersuite_ECDHE_RSA_WITH_AES_256_GCM_SHA384, 0xC030
        )
        self.assertEqual(
            Security.tls_ciphersuite_ECDHE_RSA_WITH_CHACHA20_POLY1305_SHA256, 0xCCA8
        )
        self.assertEqual(
            Security.tls_ciphersuite_ECDHE_ECDSA_WITH_CHACHA20_POLY1305_SHA256, 0xCCA9
        )
        self.assertEqual(Security.tls_ciphersuite_AES_128_GCM_SHA256, 0x1301)
        self.assertEqual(Security.tls_ciphersuite_AES_256_GCM_SHA384, 0x1302)
        self.assertEqual(Security.tls_ciphersuite_CHACHA20_POLY1305_SHA256, 0x1303)

        self.assertEqual(Security.tls_ciphersuite_group_default, 0)
        self.assertEqual(Security.tls_ciphersuite_group_compatibility, 1)
        self.assertEqual(Security.tls_ciphersuite_group_legacy, 2)
        self.assertEqual(Security.tls_ciphersuite_group_ats, 3)
        self.assertEqual(Security.tls_ciphersuite_group_ats_compatibility, 4)

        self.assertEqual(Security.kSSLProtocolUnknown, 0)
        self.assertEqual(Security.kTLSProtocol1, 4)
        self.assertEqual(Security.kTLSProtocol11, 7)
        self.assertEqual(Security.kTLSProtocol12, 8)
        self.assertEqual(Security.kDTLSProtocol1, 9)
        self.assertEqual(Security.kTLSProtocol13, 10)
        self.assertEqual(Security.kDTLSProtocol12, 11)
        self.assertEqual(Security.kTLSProtocolMaxSupported, 999)
        self.assertEqual(Security.kSSLProtocol2, 1)
        self.assertEqual(Security.kSSLProtocol3, 2)
        self.assertEqual(Security.kSSLProtocol3Only, 3)
        self.assertEqual(Security.kTLSProtocol1Only, 5)
        self.assertEqual(Security.kSSLProtocolAll, 6)

    @min_os_level("10.14")
    def test_functions(self):
        self.assertResultIsRetained(Security.sec_trust_create)
        self.assertResultIsCFRetained(Security.sec_trust_copy_ref)

        self.assertResultIsRetained(Security.sec_identity_create)
        self.assertResultIsRetained(Security.sec_identity_create_with_certificates)
        self.assertResultIsCFRetained(Security.sec_identity_copy_ref)
        self.assertResultIsCFRetained(Security.sec_identity_copy_certificates_ref)

        self.assertResultIsRetained(Security.sec_certificate_create)
        self.assertResultIsCFRetained(Security.sec_certificate_copy_ref)

    @min_os_level("10.15")
    def test_functions10_15(self):
        self.assertArgIsBlock(Security.sec_identity_access_certificates, 1, b"v@")



import Security
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestSecDecodeTransform(TestCase):
    @min_os_level("10.7")
    def test_constants(self):
        self.assertIsInstance(Security.kSecDecodeTypeAttribute, str)

    @min_os_level("10.7")
    def test_functions10_7(self):
        self.assertResultHasType(Security.SecDecodeTransformCreate, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecDecodeTransformCreate)
        self.assertArgHasType(Security.SecDecodeTransformCreate, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecDecodeTransformCreate, 1, objc._C_OUT + objc._C_PTR + objc._C_ID
        )



import Security
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc

SecTrustCallback = b"v@I"
SecTrustWithErrorCallback = b"v@B@"


class TestSecTrusted(TestCase):
    def test_types(self):
        self.assertIsCFType(Security.SecTrustRef)

    def test_constants(self):
        self.assertEqual(Security.kSecTrustResultInvalid, 0)
        self.assertEqual(Security.kSecTrustResultProceed, 1)
        self.assertEqual(Security.kSecTrustResultConfirm, 2)
        self.assertEqual(Security.kSecTrustResultDeny, 3)
        self.assertEqual(Security.kSecTrustResultUnspecified, 4)
        self.assertEqual(Security.kSecTrustResultRecoverableTrustFailure, 5)
        self.assertEqual(Security.kSecTrustResultFatalTrustFailure, 6)
        self.assertEqual(Security.kSecTrustResultOtherError, 7)

        self.assertEqual(Security.kSecTrustOptionAllowExpired, 0x00000001)
        self.assertEqual(Security.kSecTrustOptionLeafIsCA, 0x00000002)
        self.assertEqual(Security.kSecTrustOptionFetchIssuerFromNet, 0x00000004)
        self.assertEqual(Security.kSecTrustOptionAllowExpiredRoot, 0x00000008)
        self.assertEqual(Security.kSecTrustOptionRequireRevPerCert, 0x00000010)
        self.assertEqual(Security.kSecTrustOptionUseTrustSettings, 0x00000020)
        self.assertEqual(Security.kSecTrustOptionImplicitAnchors, 0x00000040)

    @min_os_level("10.7")
    def test_constants10_7(self):
        self.assertIsInstance(Security.kSecPropertyTypeTitle, str)
        self.assertIsInstance(Security.kSecPropertyTypeError, str)

    @min_os_level("10.9")
    def test_constants10_9(self):
        self.assertIsInstance(Security.kSecTrustEvaluationDate, str)
        self.assertIsInstance(Security.kSecTrustExtendedValidation, str)
        self.assertIsInstance(Security.kSecTrustOrganizationName, str)
        self.assertIsInstance(Security.kSecTrustResultValue, str)
        self.assertIsInstance(Security.kSecTrustRevocationChecked, str)
        self.assertIsInstance(Security.kSecTrustRevocationValidUntilDate, str)

    @min_os_level("10.11")
    def test_constants10_11(self):
        self.assertIsInstance(Security.kSecTrustCertificateTransparency, str)

    @min_os_level("10.12")
    def test_constants10_12(self):
        self.assertIsInstance(Security.kSecTrustCertificateTransparencyWhiteList, str)

    def test_functions(self):
        self.assertIsInstance(Security.SecTrustGetTypeID(), int)

        self.assertResultHasType(Security.SecTrustCreateWithCertificates, objc._C_INT)
        self.assertArgHasType(Security.SecTrustCreateWithCertificates, 0, objc._C_ID)
        self.assertArgHasType(Security.SecTrustCreateWithCertificates, 1, objc._C_ID)
        self.assertArgHasType(
            Security.SecTrustCreateWithCertificates,
            2,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )
        self.assertArgIsCFRetained(Security.SecTrustCreateWithCertificates, 2)

        self.assertResultHasType(Security.SecTrustSetPolicies, objc._C_INT)
        self.assertArgHasType(Security.SecTrustSetPolicies, 0, objc._C_ID)
        self.assertArgHasType(Security.SecTrustSetPolicies, 1, objc._C_ID)

        self.assertResultHasType(Security.SecTrustCopyPolicies, objc._C_INT)
        self.assertArgHasType(Security.SecTrustCopyPolicies, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecTrustCopyPolicies, 1, objc._C_OUT + objc._C_PTR + objc._C_ID
        )
        self.assertArgIsCFRetained(Security.SecTrustCopyPolicies, 1)

        self.assertResultHasType(Security.SecTrustSetAnchorCertificates, objc._C_INT)
        self.assertArgHasType(Security.SecTrustSetAnchorCertificates, 0, objc._C_ID)
        self.assertArgHasType(Security.SecTrustSetAnchorCertificates, 1, objc._C_ID)

        self.assertResultHasType(
            Security.SecTrustSetAnchorCertificatesOnly, objc._C_INT
        )
        self.assertArgHasType(Security.SecTrustSetAnchorCertificatesOnly, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecTrustSetAnchorCertificatesOnly, 1, objc._C_NSBOOL
        )

        self.assertResultHasType(
            Security.SecTrustCopyCustomAnchorCertificates, objc._C_INT
        )
        self.assertArgHasType(
            Security.SecTrustCopyCustomAnchorCertificates, 0, objc._C_ID
        )
        self.assertArgHasType(
            Security.SecTrustCopyCustomAnchorCertificates,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )
        self.assertArgIsCFRetained(Security.SecTrustCopyCustomAnchorCertificates, 1)

        self.assertResultHasType(Security.SecTrustSetVerifyDate, objc._C_INT)
        self.assertArgHasType(Security.SecTrustSetVerifyDate, 0, objc._C_ID)
        self.assertArgHasType(Security.SecTrustSetVerifyDate, 1, objc._C_ID)

        self.assertResultHasType(Security.SecTrustGetVerifyTime, objc._C_DBL)
        self.assertArgHasType(Security.SecTrustGetVerifyTime, 0, objc._C_ID)

        self.assertResultHasType(Security.SecTrustSetKeychains, objc._C_INT)
        self.assertArgHasType(Security.SecTrustSetKeychains, 0, objc._C_ID)
        self.assertArgHasType(Security.SecTrustSetKeychains, 1, objc._C_ID)

        self.assertFalse(hasattr(Security, "SecTrustSetParameters"))
        self.assertFalse(hasattr(Security, "SecTrustGetResult"))
        self.assertFalse(hasattr(Security, "SecTrustGetCssmResult"))
        self.assertFalse(hasattr(Security, "SecTrustGetCssmResultCode"))
        self.assertFalse(hasattr(Security, "SecTrustGetTPHandle"))

        self.assertResultHasType(Security.SecTrustCopyAnchorCertificates, objc._C_INT)
        self.assertArgHasType(
            Security.SecTrustCopyAnchorCertificates,
            0,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )
        self.assertArgIsCFRetained(Security.SecTrustCopyAnchorCertificates, 0)

    @min_os_level("10.7")
    def test_functions_10_7(self):
        self.assertResultHasType(Security.SecTrustEvaluate, objc._C_INT)
        self.assertArgHasType(Security.SecTrustEvaluate, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecTrustEvaluate, 1, objc._C_OUT + objc._C_PTR + objc._C_UINT
        )

        self.assertResultHasType(Security.SecTrustEvaluateAsync, objc._C_INT)
        self.assertArgHasType(Security.SecTrustEvaluateAsync, 0, objc._C_ID)
        self.assertArgHasType(Security.SecTrustEvaluateAsync, 1, b"^{dispatch_queue_s}")
        self.assertArgIsBlock(Security.SecTrustEvaluateAsync, 2, SecTrustCallback)

        self.assertResultHasType(Security.SecTrustGetTrustResult, objc._C_INT)
        self.assertArgHasType(Security.SecTrustGetTrustResult, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecTrustGetTrustResult, 1, objc._C_OUT + objc._C_PTR + objc._C_UINT
        )

        self.assertResultHasType(Security.SecTrustCopyPublicKey, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecTrustCopyPublicKey)
        self.assertArgHasType(Security.SecTrustCopyPublicKey, 0, objc._C_ID)

        self.assertResultHasType(
            Security.SecTrustGetCertificateCount, objc._C_NSInteger
        )
        self.assertArgHasType(Security.SecTrustGetCertificateCount, 0, objc._C_ID)

        self.assertResultHasType(Security.SecTrustGetCertificateAtIndex, objc._C_ID)
        self.assertArgHasType(Security.SecTrustGetCertificateAtIndex, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecTrustGetCertificateAtIndex, 1, objc._C_NSInteger
        )

        self.assertResultHasType(Security.SecTrustCopyProperties, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecTrustCopyProperties)
        self.assertArgHasType(Security.SecTrustCopyProperties, 0, objc._C_ID)

        self.assertResultHasType(Security.SecTrustSetOptions, objc._C_INT)
        self.assertArgHasType(Security.SecTrustSetOptions, 0, objc._C_ID)
        self.assertArgHasType(Security.SecTrustSetOptions, 1, objc._C_UINT)

    @min_os_level("10.9")
    def test_functions_10_9(self):
        self.assertResultHasType(Security.SecTrustSetNetworkFetchAllowed, objc._C_INT)
        self.assertArgHasType(Security.SecTrustSetNetworkFetchAllowed, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecTrustSetNetworkFetchAllowed, 1, objc._C_NSBOOL
        )

        self.assertResultHasType(Security.SecTrustGetNetworkFetchAllowed, objc._C_INT)
        self.assertArgHasType(Security.SecTrustGetNetworkFetchAllowed, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecTrustGetNetworkFetchAllowed,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_NSBOOL,
        )

        self.assertResultHasType(Security.SecTrustCopyExceptions, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecTrustCopyExceptions)
        self.assertArgHasType(Security.SecTrustCopyExceptions, 0, objc._C_ID)

        self.assertResultHasType(Security.SecTrustSetExceptions, objc._C_BOOL)
        self.assertArgHasType(Security.SecTrustSetExceptions, 0, objc._C_ID)
        self.assertArgHasType(Security.SecTrustSetExceptions, 1, objc._C_ID)

        self.assertResultHasType(Security.SecTrustCopyResult, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecTrustCopyResult)
        self.assertArgHasType(Security.SecTrustCopyResult, 0, objc._C_ID)

        self.assertResultHasType(Security.SecTrustSetOCSPResponse, objc._C_INT)
        self.assertArgHasType(Security.SecTrustSetOCSPResponse, 0, objc._C_ID)
        self.assertArgHasType(Security.SecTrustSetOCSPResponse, 1, objc._C_ID)

    @min_os_level("10.14")
    def test_functions_10_14(self):
        self.assertResultHasType(Security.SecTrustEvaluateWithError, objc._C_BOOL)
        self.assertArgHasType(Security.SecTrustEvaluateWithError, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecTrustEvaluateWithError,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )

    @min_os_level("10.14.2")
    def test_functions_10_14_2(self):
        Security.SecTrustSetSignedCertificateTimestamps

    @min_os_level("10.15")
    def test_functions_10_15(self):
        self.assertArgIsBlock(
            Security.SecTrustEvaluateAsyncWithError, 2, SecTrustWithErrorCallback
        )

    @min_os_level("10.16")
    def test_functions10_16(self):
        self.assertResultIsCFRetained(Security.SecTrustCopyKey)



import Security
from PyObjCTools.TestSupport import TestCase


class Testoidsalg(TestCase):
    def test_unsuppported(self):
        self.assertFalse(hasattr(Security, "CSSMOID_MD2"))
        self.assertFalse(hasattr(Security, "CSSMOID_MD4"))
        self.assertFalse(hasattr(Security, "CSSMOID_MD5"))
        self.assertFalse(hasattr(Security, "CSSMOID_RSA"))
        self.assertFalse(hasattr(Security, "CSSMOID_MD2WithRSA"))
        self.assertFalse(hasattr(Security, "CSSMOID_MD4WithRSA"))
        self.assertFalse(hasattr(Security, "CSSMOID_MD5WithRSA"))
        self.assertFalse(hasattr(Security, "CSSMOID_SHA1WithRSA"))
        self.assertFalse(hasattr(Security, "CSSMOID_SHA224WithRSA"))
        self.assertFalse(hasattr(Security, "CSSMOID_SHA256WithRSA"))
        self.assertFalse(hasattr(Security, "CSSMOID_SHA384WithRSA"))
        self.assertFalse(hasattr(Security, "CSSMOID_SHA512WithRSA"))
        self.assertFalse(hasattr(Security, "CSSMOID_SHA1WithRSA_OIW"))
        self.assertFalse(hasattr(Security, "CSSMOID_RSAWithOAEP"))
        self.assertFalse(hasattr(Security, "CSSMOID_OAEP_MGF1"))
        self.assertFalse(hasattr(Security, "CSSMOID_OAEP_ID_PSPECIFIED"))
        self.assertFalse(hasattr(Security, "CSSMOID_DES_CBC"))
        self.assertFalse(hasattr(Security, "CSSMOID_ANSI_DH_PUB_NUMBER"))
        self.assertFalse(hasattr(Security, "CSSMOID_ANSI_DH_STATIC"))
        self.assertFalse(hasattr(Security, "CSSMOID_ANSI_DH_ONE_FLOW"))
        self.assertFalse(hasattr(Security, "CSSMOID_ANSI_DH_EPHEM"))
        self.assertFalse(hasattr(Security, "CSSMOID_ANSI_DH_HYBRID1"))
        self.assertFalse(hasattr(Security, "CSSMOID_ANSI_DH_HYBRID2"))
        self.assertFalse(hasattr(Security, "CSSMOID_ANSI_DH_HYBRID_ONEFLOW"))
        self.assertFalse(hasattr(Security, "CSSMOID_ANSI_MQV1"))
        self.assertFalse(hasattr(Security, "CSSMOID_ANSI_MQV2"))
        self.assertFalse(hasattr(Security, "CSSMOID_ANSI_DH_STATIC_SHA1"))
        self.assertFalse(hasattr(Security, "CSSMOID_ANSI_DH_ONE_FLOW_SHA1"))
        self.assertFalse(hasattr(Security, "CSSMOID_ANSI_DH_EPHEM_SHA1"))
        self.assertFalse(hasattr(Security, "CSSMOID_ANSI_DH_HYBRID1_SHA1"))
        self.assertFalse(hasattr(Security, "CSSMOID_ANSI_DH_HYBRID2_SHA1"))
        self.assertFalse(hasattr(Security, "CSSMOID_ANSI_MQV1_SHA1"))
        self.assertFalse(hasattr(Security, "CSSMOID_ANSI_MQV2_SHA1"))
        self.assertFalse(hasattr(Security, "CSSMOID_PKCS3"))
        self.assertFalse(hasattr(Security, "CSSMOID_DH"))
        self.assertFalse(hasattr(Security, "CSSMOID_DSA  				// BSAFE only"))
        self.assertFalse(hasattr(Security, "CSSMOID_DSA_CMS  			// X509/CMS"))
        self.assertFalse(hasattr(Security, "CSSMOID_DSA_JDK  			// JDK 1.1"))
        self.assertFalse(hasattr(Security, "CSSMOID_SHA1WithDSA  		// BSAFE"))
        self.assertFalse(hasattr(Security, "CSSMOID_SHA1WithDSA_CMS  	// X509/CMS"))
        self.assertFalse(hasattr(Security, "CSSMOID_SHA1WithDSA_JDK  	// JDK 1.1"))
        self.assertFalse(hasattr(Security, "CSSMOID_SHA1"))
        self.assertFalse(hasattr(Security, "CSSMOID_SHA224"))
        self.assertFalse(hasattr(Security, "CSSMOID_SHA256"))
        self.assertFalse(hasattr(Security, "CSSMOID_SHA384"))
        self.assertFalse(hasattr(Security, "CSSMOID_SHA512"))
        self.assertFalse(hasattr(Security, "CSSMOID_ecPublicKey"))
        self.assertFalse(hasattr(Security, "CSSMOID_ECDSA_WithSHA1"))
        self.assertFalse(hasattr(Security, "CSSMOID_ECDSA_WithSHA224"))
        self.assertFalse(hasattr(Security, "CSSMOID_ECDSA_WithSHA256"))
        self.assertFalse(hasattr(Security, "CSSMOID_ECDSA_WithSHA384"))
        self.assertFalse(hasattr(Security, "CSSMOID_ECDSA_WithSHA512"))
        self.assertFalse(hasattr(Security, "CSSMOID_ECDSA_WithSpecified"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_ISIGN"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_X509_BASIC"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_TP_SSL"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_TP_LOCAL_CERT_GEN"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_TP_CSR_GEN"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_TP_REVOCATION_CRL"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_TP_REVOCATION_OCSP"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_TP_SMIME"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_TP_EAP"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_TP_CODE_SIGN"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_TP_SW_UPDATE_SIGNING"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_TP_IP_SEC"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_TP_ICHAT"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_TP_RESOURCE_SIGN"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_TP_PKINIT_CLIENT"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_TP_PKINIT_SERVER"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_TP_CODE_SIGNING"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_TP_PACKAGE_SIGNING"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_TP_MACAPPSTORE_RECEIPT"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_TP_APPLEID_SHARING"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_TP_TIMESTAMPING"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_TP_REVOCATION"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_TP_PASSBOOK_SIGNING"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_TP_MOBILE_STORE"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_TP_ESCROW_SERVICE"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_TP_PROFILE_SIGNING"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_TP_QA_PROFILE_SIGNING"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_TP_TEST_MOBILE_STORE"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_TP_PCS_ESCROW_SERVICE"))
        self.assertFalse(
            hasattr(Security, "CSSMOID_APPLE_TP_PROVISIONING_PROFILE_SIGNING")
        )
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_FEE"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_ASC"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_FEE_MD5"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_FEE_SHA1"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_FEED"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_FEEDEXP"))
        self.assertFalse(hasattr(Security, "CSSMOID_APPLE_ECDSA"))
        self.assertFalse(hasattr(Security, "CSSMOID_DOTMAC_CERT"))
        self.assertFalse(hasattr(Security, "CSSMOID_DOTMAC_CERT_REQ"))
        self.assertFalse(hasattr(Security, "CSSMOID_DOTMAC_CERT_REQ_IDENTITY"))
        self.assertFalse(hasattr(Security, "CSSMOID_DOTMAC_CERT_REQ_EMAIL_SIGN"))
        self.assertFalse(hasattr(Security, "CSSMOID_DOTMAC_CERT_REQ_EMAIL_ENCRYPT"))
        self.assertFalse(hasattr(Security, "CSSMOID_DOTMAC_CERT_REQ_ARCHIVE_LIST"))
        self.assertFalse(hasattr(Security, "CSSMOID_DOTMAC_CERT_REQ_ARCHIVE_STORE"))
        self.assertFalse(hasattr(Security, "CSSMOID_DOTMAC_CERT_REQ_ARCHIVE_FETCH"))
        self.assertFalse(hasattr(Security, "CSSMOID_DOTMAC_CERT_REQ_ARCHIVE_REMOVE"))
        self.assertFalse(hasattr(Security, "CSSMOID_DOTMAC_CERT_REQ_SHARED_SERVICES"))
        self.assertFalse(hasattr(Security, "CSSMOID_DOTMAC_CERT_REQ_VALUE_USERNAME"))
        self.assertFalse(hasattr(Security, "CSSMOID_DOTMAC_CERT_REQ_VALUE_PASSWORD"))
        self.assertFalse(hasattr(Security, "CSSMOID_DOTMAC_CERT_REQ_VALUE_HOSTNAME"))
        self.assertFalse(hasattr(Security, "CSSMOID_DOTMAC_CERT_REQ_VALUE_RENEW"))
        self.assertFalse(hasattr(Security, "CSSMOID_DOTMAC_CERT_REQ_VALUE_ASYNC"))
        self.assertFalse(hasattr(Security, "CSSMOID_DOTMAC_CERT_REQ_VALUE_IS_PENDING"))
        self.assertFalse(hasattr(Security, "CSSMOID_PKCS5_DIGEST_ALG"))
        self.assertFalse(hasattr(Security, "CSSMOID_PKCS5_ENCRYPT_ALG"))
        self.assertFalse(hasattr(Security, "CSSMOID_PKCS5_HMAC_SHA1"))
        self.assertFalse(hasattr(Security, "CSSMOID_PKCS5_pbeWithMD2AndDES"))
        self.assertFalse(hasattr(Security, "CSSMOID_PKCS5_pbeWithMD2AndRC2"))
        self.assertFalse(hasattr(Security, "CSSMOID_PKCS5_pbeWithMD5AndDES"))
        self.assertFalse(hasattr(Security, "CSSMOID_PKCS5_pbeWithMD5AndRC2"))
        self.assertFalse(hasattr(Security, "CSSMOID_PKCS5_pbeWithSHA1AndDES"))
        self.assertFalse(hasattr(Security, "CSSMOID_PKCS5_pbeWithSHA1AndRC2"))
        self.assertFalse(hasattr(Security, "CSSMOID_PKCS5_PBKDF2"))
        self.assertFalse(hasattr(Security, "CSSMOID_PKCS5_PBES2"))
        self.assertFalse(hasattr(Security, "CSSMOID_PKCS5_PBMAC1"))
        self.assertFalse(hasattr(Security, "CSSMOID_PKCS5_RC2_CBC"))
        self.assertFalse(hasattr(Security, "CSSMOID_PKCS5_DES_EDE3_CBC"))
        self.assertFalse(hasattr(Security, "CSSMOID_PKCS5_RC5_CBC"))
        self.assertFalse(hasattr(Security, "CSSMOID_PKCS12_pbeWithSHAAnd128BitRC4"))
        self.assertFalse(hasattr(Security, "CSSMOID_PKCS12_pbeWithSHAAnd40BitRC4"))
        self.assertFalse(hasattr(Security, "CSSMOID_PKCS12_pbeWithSHAAnd3Key3DESCBC"))
        self.assertFalse(hasattr(Security, "CSSMOID_PKCS12_pbeWithSHAAnd2Key3DESCBC"))
        self.assertFalse(hasattr(Security, "CSSMOID_PKCS12_pbeWithSHAAnd128BitRC2CBC"))
        self.assertFalse(hasattr(Security, "CSSMOID_PKCS12_pbewithSHAAnd40BitRC2CBC"))



import Security
from PyObjCTools.TestSupport import TestCase, min_os_level, expectedFailure


class TestSecCertificateOIDs(TestCase):
    def test_constants(self):
        self.assertIsInstance(Security.kSecOIDADC_CERT_POLICY, str)
        self.assertIsInstance(Security.kSecOIDAPPLE_CERT_POLICY, str)
        self.assertIsInstance(Security.kSecOIDAPPLE_EKU_CODE_SIGNING, str)
        self.assertIsInstance(Security.kSecOIDAPPLE_EKU_CODE_SIGNING_DEV, str)
        self.assertIsInstance(Security.kSecOIDAPPLE_EKU_ICHAT_ENCRYPTION, str)
        self.assertIsInstance(Security.kSecOIDAPPLE_EKU_ICHAT_SIGNING, str)
        self.assertIsInstance(Security.kSecOIDAPPLE_EKU_RESOURCE_SIGNING, str)
        self.assertIsInstance(Security.kSecOIDAPPLE_EKU_SYSTEM_IDENTITY, str)
        self.assertIsInstance(Security.kSecOIDAPPLE_EXTENSION, str)
        self.assertIsInstance(Security.kSecOIDAPPLE_EXTENSION_ADC_APPLE_SIGNING, str)
        self.assertIsInstance(Security.kSecOIDAPPLE_EXTENSION_ADC_DEV_SIGNING, str)
        self.assertIsInstance(Security.kSecOIDAPPLE_EXTENSION_APPLE_SIGNING, str)
        self.assertIsInstance(Security.kSecOIDAPPLE_EXTENSION_CODE_SIGNING, str)
        self.assertIsInstance(Security.kSecOIDAuthorityInfoAccess, str)
        self.assertIsInstance(Security.kSecOIDAuthorityKeyIdentifier, str)
        self.assertIsInstance(Security.kSecOIDBasicConstraints, str)
        self.assertIsInstance(Security.kSecOIDBiometricInfo, str)
        self.assertIsInstance(Security.kSecOIDCSSMKeyStruct, str)
        self.assertIsInstance(Security.kSecOIDCertIssuer, str)
        self.assertIsInstance(Security.kSecOIDCertificatePolicies, str)
        self.assertIsInstance(Security.kSecOIDClientAuth, str)
        self.assertIsInstance(Security.kSecOIDCollectiveStateProvinceName, str)
        self.assertIsInstance(Security.kSecOIDCollectiveStreetAddress, str)
        self.assertIsInstance(Security.kSecOIDCommonName, str)
        self.assertIsInstance(Security.kSecOIDCountryName, str)
        self.assertIsInstance(Security.kSecOIDCrlDistributionPoints, str)
        self.assertIsInstance(Security.kSecOIDCrlNumber, str)
        self.assertIsInstance(Security.kSecOIDCrlReason, str)
        self.assertIsInstance(Security.kSecOIDDOTMAC_CERT_EMAIL_ENCRYPT, str)
        self.assertIsInstance(Security.kSecOIDDOTMAC_CERT_EMAIL_SIGN, str)
        self.assertIsInstance(Security.kSecOIDDOTMAC_CERT_EXTENSION, str)
        self.assertIsInstance(Security.kSecOIDDOTMAC_CERT_IDENTITY, str)
        self.assertIsInstance(Security.kSecOIDDOTMAC_CERT_POLICY, str)
        self.assertIsInstance(Security.kSecOIDDeltaCrlIndicator, str)
        self.assertIsInstance(Security.kSecOIDDescription, str)
        self.assertIsInstance(Security.kSecOIDEKU_IPSec, str)
        self.assertIsInstance(Security.kSecOIDEmailAddress, str)
        self.assertIsInstance(Security.kSecOIDEmailProtection, str)
        self.assertIsInstance(Security.kSecOIDExtendedKeyUsage, str)
        self.assertIsInstance(Security.kSecOIDExtendedKeyUsageAny, str)
        self.assertIsInstance(Security.kSecOIDExtendedUseCodeSigning, str)
        self.assertIsInstance(Security.kSecOIDGivenName, str)
        self.assertIsInstance(Security.kSecOIDHoldInstructionCode, str)
        self.assertIsInstance(Security.kSecOIDInvalidityDate, str)
        self.assertIsInstance(Security.kSecOIDIssuerAltName, str)
        self.assertIsInstance(Security.kSecOIDIssuingDistributionPoint, str)
        self.assertIsInstance(Security.kSecOIDIssuingDistributionPoints, str)
        self.assertIsInstance(Security.kSecOIDKERBv5_PKINIT_KP_CLIENT_AUTH, str)
        self.assertIsInstance(Security.kSecOIDKERBv5_PKINIT_KP_KDC, str)
        self.assertIsInstance(Security.kSecOIDKeyUsage, str)
        self.assertIsInstance(Security.kSecOIDLocalityName, str)
        self.assertIsInstance(Security.kSecOIDMS_NTPrincipalName, str)
        self.assertIsInstance(Security.kSecOIDMicrosoftSGC, str)
        self.assertIsInstance(Security.kSecOIDNameConstraints, str)
        self.assertIsInstance(Security.kSecOIDNetscapeCertSequence, str)
        self.assertIsInstance(Security.kSecOIDNetscapeCertType, str)
        self.assertIsInstance(Security.kSecOIDNetscapeSGC, str)
        self.assertIsInstance(Security.kSecOIDOCSPSigning, str)
        self.assertIsInstance(Security.kSecOIDOrganizationName, str)
        self.assertIsInstance(Security.kSecOIDOrganizationalUnitName, str)
        self.assertIsInstance(Security.kSecOIDPolicyConstraints, str)
        self.assertIsInstance(Security.kSecOIDPolicyMappings, str)
        self.assertIsInstance(Security.kSecOIDPrivateKeyUsagePeriod, str)
        self.assertIsInstance(Security.kSecOIDQC_Statements, str)
        self.assertIsInstance(Security.kSecOIDSerialNumber, str)
        self.assertIsInstance(Security.kSecOIDServerAuth, str)
        self.assertIsInstance(Security.kSecOIDStateProvinceName, str)
        self.assertIsInstance(Security.kSecOIDStreetAddress, str)
        self.assertIsInstance(Security.kSecOIDSubjectAltName, str)
        self.assertIsInstance(Security.kSecOIDSubjectDirectoryAttributes, str)
        self.assertIsInstance(Security.kSecOIDSubjectEmailAddress, str)
        self.assertIsInstance(Security.kSecOIDSubjectInfoAccess, str)
        self.assertIsInstance(Security.kSecOIDSubjectKeyIdentifier, str)
        self.assertIsInstance(Security.kSecOIDSubjectPicture, str)
        self.assertIsInstance(Security.kSecOIDSubjectSignatureBitmap, str)
        self.assertIsInstance(Security.kSecOIDSurname, str)
        self.assertIsInstance(Security.kSecOIDTimeStamping, str)
        self.assertIsInstance(Security.kSecOIDTitle, str)
        self.assertIsInstance(Security.kSecOIDUseExemptions, str)
        self.assertIsInstance(Security.kSecOIDX509V1CertificateIssuerUniqueId, str)
        self.assertIsInstance(Security.kSecOIDX509V1CertificateSubjectUniqueId, str)
        self.assertIsInstance(Security.kSecOIDX509V1IssuerName, str)
        self.assertIsInstance(Security.kSecOIDX509V1IssuerNameCStruct, str)
        self.assertIsInstance(Security.kSecOIDX509V1IssuerNameLDAP, str)
        self.assertIsInstance(Security.kSecOIDX509V1IssuerNameStd, str)
        self.assertIsInstance(Security.kSecOIDX509V1SerialNumber, str)
        self.assertIsInstance(Security.kSecOIDX509V1Signature, str)
        self.assertIsInstance(Security.kSecOIDX509V1SignatureAlgorithm, str)
        self.assertIsInstance(Security.kSecOIDX509V1SignatureAlgorithmParameters, str)
        self.assertIsInstance(Security.kSecOIDX509V1SignatureAlgorithmTBS, str)
        self.assertIsInstance(Security.kSecOIDX509V1SignatureCStruct, str)
        self.assertIsInstance(Security.kSecOIDX509V1SignatureStruct, str)
        self.assertIsInstance(Security.kSecOIDX509V1SubjectName, str)
        self.assertIsInstance(Security.kSecOIDX509V1SubjectNameCStruct, str)
        self.assertIsInstance(Security.kSecOIDX509V1SubjectNameLDAP, str)
        self.assertIsInstance(Security.kSecOIDX509V1SubjectNameStd, str)
        self.assertIsInstance(Security.kSecOIDX509V1SubjectPublicKey, str)
        self.assertIsInstance(Security.kSecOIDX509V1SubjectPublicKeyAlgorithm, str)
        self.assertIsInstance(
            Security.kSecOIDX509V1SubjectPublicKeyAlgorithmParameters, str
        )
        self.assertIsInstance(Security.kSecOIDX509V1SubjectPublicKeyCStruct, str)
        self.assertIsInstance(Security.kSecOIDX509V1ValidityNotAfter, str)
        self.assertIsInstance(Security.kSecOIDX509V1ValidityNotBefore, str)
        self.assertIsInstance(Security.kSecOIDX509V1Version, str)
        self.assertIsInstance(Security.kSecOIDX509V3Certificate, str, str)
        self.assertIsInstance(Security.kSecOIDX509V3CertificateCStruct, str)
        self.assertIsInstance(Security.kSecOIDX509V3CertificateExtensionCStruct, str)
        self.assertIsInstance(Security.kSecOIDX509V3CertificateExtensionCritical, str)
        self.assertIsInstance(Security.kSecOIDX509V3CertificateExtensionId, str)
        self.assertIsInstance(Security.kSecOIDX509V3CertificateExtensionStruct, str)
        self.assertIsInstance(Security.kSecOIDX509V3CertificateExtensionType, str)
        self.assertIsInstance(Security.kSecOIDX509V3CertificateExtensionValue, str)
        self.assertIsInstance(Security.kSecOIDX509V3CertificateExtensionsCStruct, str)
        self.assertIsInstance(Security.kSecOIDX509V3CertificateExtensionsStruct, str)
        self.assertIsInstance(Security.kSecOIDX509V3CertificateNumberOfExtensions, str)
        self.assertIsInstance(Security.kSecOIDX509V3SignedCertificate, str)
        self.assertIsInstance(Security.kSecOIDX509V3SignedCertificateCStruct, str)

    @expectedFailure
    @min_os_level("10.7")
    def test_constants_missing(self):
        self.assertIsInstance(Security.kSecOIDAPPLE_EXTENSION_INTERMEDIATE_MARKER, str)
        self.assertIsInstance(Security.kSecOIDAPPLE_EXTENSION_WWDR_INTERMEDIATE, str)
        self.assertIsInstance(Security.kSecOIDAPPLE_EXTENSION_ITMS_INTERMEDIATE, str)
        self.assertIsInstance(Security.kSecOIDAPPLE_EXTENSION_AAI_INTERMEDIATE, str)
        self.assertIsInstance(Security.kSecOIDAPPLE_EXTENSION_APPLEID_INTERMEDIATE, str)

    @min_os_level("10.8")
    def test_constants_10_8(self):
        self.assertIsInstance(Security.kSecOIDSRVName, str)



import Security
from PyObjCTools.TestSupport import TestCase, min_os_level, expectedFailure
import objc


class TestSecEncryptTransform(TestCase):
    def test_constants(self):
        self.assertIsInstance(Security.kSecPaddingNoneKey, str)
        self.assertIsInstance(Security.kSecPaddingPKCS1Key, str)
        self.assertIsInstance(Security.kSecPaddingPKCS5Key, str)
        self.assertIsInstance(Security.kSecPaddingPKCS7Key, str)
        self.assertIsInstance(Security.kSecPaddingOAEPKey, str)
        self.assertIsInstance(Security.kSecModeNoneKey, str)
        self.assertIsInstance(Security.kSecModeECBKey, str)
        self.assertIsInstance(Security.kSecModeCBCKey, str)
        self.assertIsInstance(Security.kSecModeCFBKey, str)
        self.assertIsInstance(Security.kSecModeOFBKey, str)
        self.assertIsInstance(Security.kSecEncryptKey, str)
        self.assertIsInstance(Security.kSecPaddingKey, str)
        self.assertIsInstance(Security.kSecIVKey, str)
        self.assertIsInstance(Security.kSecEncryptionMode, str)

    @min_os_level("10.8")
    def test_constants10_8(self):
        self.assertIsInstance(Security.kSecOAEPEncodingParametersAttributeName, str)

    @expectedFailure
    @min_os_level("10.8")
    def test_constants10_8_missing(self):
        self.assertIsInstance(Security.kSecOAEPMessageLengthAttributeName, str)
        self.assertIsInstance(Security.kSecOAEPMGF1DigestAlgorithmAttributeName, str)

    @min_os_level("10.7")
    def test_functions(self):
        self.assertResultHasType(Security.SecEncryptTransformCreate, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecEncryptTransformCreate)
        self.assertArgHasType(Security.SecEncryptTransformCreate, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecEncryptTransformCreate,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )

        self.assertResultHasType(Security.SecDecryptTransformCreate, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecDecryptTransformCreate)
        self.assertArgHasType(Security.SecDecryptTransformCreate, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecDecryptTransformCreate,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )

    @expectedFailure
    @min_os_level("10.7")
    def test_functions_missing(self):
        # On 10.13.4 (beta) both functions are found, but crash...
        return
        self.assertIsInstance(Security.SecDecryptTransformGetTypeID(), int)
        self.assertIsInstance(Security.SecEncryptTransformGetTypeID(), int)



import Security
from PyObjCTools.TestSupport import TestCase
import objc


class TestSecTrustedApplication(TestCase):
    def test_types(self):
        self.assertIsCFType(Security.SecTrustedApplicationRef)

    def test_functions10_7(self):
        self.assertIsInstance(Security.SecTrustedApplicationGetTypeID(), int)

        self.assertResultHasType(
            Security.SecTrustedApplicationCreateFromPath, objc._C_INT
        )
        self.assertArgHasType(
            Security.SecTrustedApplicationCreateFromPath,
            0,
            objc._C_IN + objc._C_PTR + objc._C_CHAR_AS_TEXT,
        )
        self.assertArgIsNullTerminated(Security.SecTrustedApplicationCreateFromPath, 0)
        self.assertArgHasType(
            Security.SecTrustedApplicationCreateFromPath,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )
        self.assertArgIsCFRetained(Security.SecTrustedApplicationCreateFromPath, 1)

        self.assertResultHasType(Security.SecTrustedApplicationCopyData, objc._C_INT)
        self.assertArgHasType(Security.SecTrustedApplicationCopyData, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecTrustedApplicationCopyData,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )
        self.assertArgIsCFRetained(Security.SecTrustedApplicationCopyData, 1)

        self.assertResultHasType(Security.SecTrustedApplicationSetData, objc._C_INT)
        self.assertArgHasType(Security.SecTrustedApplicationSetData, 0, objc._C_ID)
        self.assertArgHasType(Security.SecTrustedApplicationSetData, 1, objc._C_ID)



import Security
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc

SecureDownloadTrustSetupCallback = b"i@^v"
SecureDownloadTrustEvaluateCallback = b"i@i^v"

SecKeyGeneratePairBlock = b"v@@@"


class TestSecKey(TestCase):
    def testTypes(self):
        self.assertIsCFType(Security.SecKeyRef)

    def test_constants(self):
        self.assertEqual(Security.kSecKeyKeyClass, 0)
        self.assertEqual(Security.kSecKeyPrintName, 1)
        self.assertEqual(Security.kSecKeyAlias, 2)
        self.assertEqual(Security.kSecKeyPermanent, 3)
        self.assertEqual(Security.kSecKeyPrivate, 4)
        self.assertEqual(Security.kSecKeyModifiable, 5)
        self.assertEqual(Security.kSecKeyLabel, 6)
        self.assertEqual(Security.kSecKeyApplicationTag, 7)
        self.assertEqual(Security.kSecKeyKeyCreator, 8)
        self.assertEqual(Security.kSecKeyKeyType, 9)
        self.assertEqual(Security.kSecKeyKeySizeInBits, 10)
        self.assertEqual(Security.kSecKeyEffectiveKeySize, 11)
        self.assertEqual(Security.kSecKeyStartDate, 12)
        self.assertEqual(Security.kSecKeyEndDate, 13)
        self.assertEqual(Security.kSecKeySensitive, 14)
        self.assertEqual(Security.kSecKeyAlwaysSensitive, 15)
        self.assertEqual(Security.kSecKeyExtractable, 16)
        self.assertEqual(Security.kSecKeyNeverExtractable, 17)
        self.assertEqual(Security.kSecKeyEncrypt, 18)
        self.assertEqual(Security.kSecKeyDecrypt, 19)
        self.assertEqual(Security.kSecKeyDerive, 20)
        self.assertEqual(Security.kSecKeySign, 21)
        self.assertEqual(Security.kSecKeyVerify, 22)
        self.assertEqual(Security.kSecKeySignRecover, 23)
        self.assertEqual(Security.kSecKeyVerifyRecover, 24)
        self.assertEqual(Security.kSecKeyWrap, 25)
        self.assertEqual(Security.kSecKeyUnwrap, 26)

        self.assertEqual(Security.kSecCredentialTypeDefault, 0)
        self.assertEqual(Security.kSecCredentialTypeWithUI, 1)
        self.assertEqual(Security.kSecCredentialTypeNoUI, 2)

        self.assertEqual(Security.kSecPaddingNone, 0)
        self.assertEqual(Security.kSecPaddingPKCS1, 1)
        self.assertEqual(Security.kSecPaddingOAEP, 2)
        self.assertEqual(Security.kSecPaddingSigRaw, 0x4000)
        self.assertEqual(Security.kSecPaddingPKCS1MD2, 0x8000)
        self.assertEqual(Security.kSecPaddingPKCS1MD5, 0x8001)
        self.assertEqual(Security.kSecPaddingPKCS1SHA1, 0x8002)
        self.assertEqual(Security.kSecPaddingPKCS1SHA224, 0x8003)
        self.assertEqual(Security.kSecPaddingPKCS1SHA256, 0x8004)
        self.assertEqual(Security.kSecPaddingPKCS1SHA384, 0x8005)
        self.assertEqual(Security.kSecPaddingPKCS1SHA512, 0x8006)

        self.assertEqual(Security.kSecDefaultKeySize, 0)
        self.assertEqual(Security.kSec3DES192, 192)
        self.assertEqual(Security.kSecAES128, 128)
        self.assertEqual(Security.kSecAES192, 192)
        self.assertEqual(Security.kSecAES256, 256)
        self.assertEqual(Security.kSecp192r1, 192)
        self.assertEqual(Security.kSecp256r1, 256)
        self.assertEqual(Security.kSecp384r1, 384)
        self.assertEqual(Security.kSecp521r1, 521)
        self.assertEqual(Security.kSecRSAMin, 1024)
        self.assertEqual(Security.kSecRSAMax, 4096)

        self.assertEqual(Security.kSecKeyOperationTypeSign, 0)
        self.assertEqual(Security.kSecKeyOperationTypeVerify, 1)
        self.assertEqual(Security.kSecKeyOperationTypeEncrypt, 2)
        self.assertEqual(Security.kSecKeyOperationTypeDecrypt, 3)
        self.assertEqual(Security.kSecKeyOperationTypeKeyExchange, 4)

    @min_os_level("10.8")
    def test_constants_10_8(self):
        self.assertIsInstance(Security.kSecPrivateKeyAttrs, str)
        self.assertIsInstance(Security.kSecPublicKeyAttrs, str)

    @min_os_level("10.12")
    def test_constants_10_12(self):
        self.assertIsInstance(Security.kSecKeyAlgorithmRSASignatureRaw, str)
        self.assertIsInstance(
            Security.kSecKeyAlgorithmRSASignatureDigestPKCS1v15Raw, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmRSASignatureDigestPKCS1v15SHA1, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmRSASignatureDigestPKCS1v15SHA224, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmRSASignatureDigestPKCS1v15SHA256, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmRSASignatureDigestPKCS1v15SHA384, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmRSASignatureDigestPKCS1v15SHA512, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmRSASignatureMessagePKCS1v15SHA1, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmRSASignatureMessagePKCS1v15SHA224, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmRSASignatureMessagePKCS1v15SHA256, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmRSASignatureMessagePKCS1v15SHA384, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmRSASignatureMessagePKCS1v15SHA512, str
        )
        self.assertIsInstance(Security.kSecKeyAlgorithmECDSASignatureRFC4754, str)
        self.assertIsInstance(Security.kSecKeyAlgorithmECDSASignatureDigestX962, str)
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECDSASignatureDigestX962SHA1, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECDSASignatureDigestX962SHA224, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECDSASignatureDigestX962SHA256, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECDSASignatureDigestX962SHA384, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECDSASignatureDigestX962SHA512, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECDSASignatureMessageX962SHA1, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECDSASignatureMessageX962SHA224, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECDSASignatureMessageX962SHA256, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECDSASignatureMessageX962SHA384, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECDSASignatureMessageX962SHA512, str
        )
        self.assertIsInstance(Security.kSecKeyAlgorithmRSAEncryptionRaw, str)
        self.assertIsInstance(Security.kSecKeyAlgorithmRSAEncryptionPKCS1, str)
        self.assertIsInstance(Security.kSecKeyAlgorithmRSAEncryptionOAEPSHA1, str)
        self.assertIsInstance(Security.kSecKeyAlgorithmRSAEncryptionOAEPSHA224, str)
        self.assertIsInstance(Security.kSecKeyAlgorithmRSAEncryptionOAEPSHA256, str)
        self.assertIsInstance(Security.kSecKeyAlgorithmRSAEncryptionOAEPSHA384, str)
        self.assertIsInstance(Security.kSecKeyAlgorithmRSAEncryptionOAEPSHA512, str)
        self.assertIsInstance(Security.kSecKeyAlgorithmRSAEncryptionOAEPSHA1AESGCM, str)
        self.assertIsInstance(
            Security.kSecKeyAlgorithmRSAEncryptionOAEPSHA224AESGCM, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmRSAEncryptionOAEPSHA256AESGCM, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmRSAEncryptionOAEPSHA384AESGCM, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmRSAEncryptionOAEPSHA512AESGCM, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECIESEncryptionStandardX963SHA1AESGCM, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECIESEncryptionStandardX963SHA224AESGCM, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECIESEncryptionStandardX963SHA256AESGCM, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECIESEncryptionStandardX963SHA384AESGCM, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECIESEncryptionStandardX963SHA512AESGCM, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECIESEncryptionCofactorX963SHA1AESGCM, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECIESEncryptionCofactorX963SHA224AESGCM, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECIESEncryptionCofactorX963SHA256AESGCM, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECIESEncryptionCofactorX963SHA384AESGCM, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECIESEncryptionCofactorX963SHA512AESGCM, str
        )
        self.assertIsInstance(Security.kSecKeyAlgorithmECDHKeyExchangeStandard, str)
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECDHKeyExchangeStandardX963SHA1, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECDHKeyExchangeStandardX963SHA224, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECDHKeyExchangeStandardX963SHA256, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECDHKeyExchangeStandardX963SHA384, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECDHKeyExchangeStandardX963SHA512, str
        )
        self.assertIsInstance(Security.kSecKeyAlgorithmECDHKeyExchangeCofactor, str)
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECDHKeyExchangeCofactorX963SHA1, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECDHKeyExchangeCofactorX963SHA224, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECDHKeyExchangeCofactorX963SHA256, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECDHKeyExchangeCofactorX963SHA384, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECDHKeyExchangeCofactorX963SHA512, str
        )

        self.assertIsInstance(Security.kSecKeyKeyExchangeParameterRequestedSize, str)
        self.assertIsInstance(Security.kSecKeyKeyExchangeParameterSharedInfo, str)

    @min_os_level("10.13")
    def test_constants_10_13(self):
        self.assertIsInstance(Security.kSecKeyAlgorithmRSASignatureDigestPSSSHA1, str)
        self.assertIsInstance(Security.kSecKeyAlgorithmRSASignatureDigestPSSSHA224, str)
        self.assertIsInstance(Security.kSecKeyAlgorithmRSASignatureDigestPSSSHA256, str)
        self.assertIsInstance(Security.kSecKeyAlgorithmRSASignatureDigestPSSSHA384, str)
        self.assertIsInstance(Security.kSecKeyAlgorithmRSASignatureDigestPSSSHA512, str)
        self.assertIsInstance(Security.kSecKeyAlgorithmRSASignatureMessagePSSSHA1, str)
        self.assertIsInstance(
            Security.kSecKeyAlgorithmRSASignatureMessagePSSSHA224, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmRSASignatureMessagePSSSHA256, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmRSASignatureMessagePSSSHA384, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmRSASignatureMessagePSSSHA512, str
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECIESEncryptionStandardVariableIVX963SHA224AESGCM,
            str,
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECIESEncryptionStandardVariableIVX963SHA256AESGCM,
            str,
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECIESEncryptionStandardVariableIVX963SHA384AESGCM,
            str,
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECIESEncryptionStandardVariableIVX963SHA512AESGCM,
            str,
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECIESEncryptionCofactorVariableIVX963SHA224AESGCM,
            str,
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECIESEncryptionCofactorVariableIVX963SHA256AESGCM,
            str,
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECIESEncryptionCofactorVariableIVX963SHA384AESGCM,
            str,
        )
        self.assertIsInstance(
            Security.kSecKeyAlgorithmECIESEncryptionCofactorVariableIVX963SHA512AESGCM,
            str,
        )

    def test_functions(self):
        self.assertIsInstance(Security.SecKeyGetTypeID(), int)

    @min_os_level("10.6")
    def test_functions_10_6(self):
        self.assertResultHasType(Security.SecKeyGetBlockSize, objc._C_ULNG)
        self.assertArgHasType(Security.SecKeyGetBlockSize, 0, objc._C_ID)

    @min_os_level("10.7")
    def test_functions_10_7(self):
        self.assertResultHasType(Security.SecKeyGenerateSymmetric, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecKeyGenerateSymmetric)
        self.assertArgHasType(Security.SecKeyGenerateSymmetric, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecKeyGenerateSymmetric, 1, objc._C_OUT + objc._C_PTR + objc._C_ID
        )

        self.assertResultHasType(Security.SecKeyCreateFromData, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecKeyCreateFromData)
        self.assertArgHasType(Security.SecKeyCreateFromData, 0, objc._C_ID)
        self.assertArgHasType(Security.SecKeyCreateFromData, 1, objc._C_ID)
        self.assertArgHasType(
            Security.SecKeyCreateFromData, 2, objc._C_OUT + objc._C_PTR + objc._C_ID
        )

        self.assertResultHasType(Security.SecKeyGeneratePairAsync, objc._C_VOID)
        self.assertArgHasType(Security.SecKeyGeneratePairAsync, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecKeyGeneratePairAsync, 1, b"^{dispatch_queue_s=}"
        )
        self.assertArgIsBlock(
            Security.SecKeyGeneratePairAsync, 2, SecKeyGeneratePairBlock
        )

        self.assertResultHasType(Security.SecKeyDeriveFromPassword, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecKeyDeriveFromPassword)
        self.assertArgHasType(Security.SecKeyDeriveFromPassword, 0, objc._C_ID)
        self.assertArgHasType(Security.SecKeyDeriveFromPassword, 1, objc._C_ID)
        self.assertArgHasType(
            Security.SecKeyDeriveFromPassword, 2, objc._C_OUT + objc._C_PTR + objc._C_ID
        )

        self.assertResultHasType(Security.SecKeyWrapSymmetric, objc._C_ID)
        self.assertArgHasType(Security.SecKeyWrapSymmetric, 0, objc._C_ID)
        self.assertArgHasType(Security.SecKeyWrapSymmetric, 1, objc._C_ID)
        self.assertArgHasType(Security.SecKeyWrapSymmetric, 2, objc._C_ID)
        self.assertArgHasType(
            Security.SecKeyWrapSymmetric, 3, objc._C_OUT + objc._C_PTR + objc._C_ID
        )

        self.assertResultHasType(Security.SecKeyUnwrapSymmetric, objc._C_ID)
        self.assertArgHasType(
            Security.SecKeyUnwrapSymmetric, 0, objc._C_IN + objc._C_PTR + objc._C_ID
        )
        self.assertArgHasType(Security.SecKeyUnwrapSymmetric, 1, objc._C_ID)
        self.assertArgHasType(Security.SecKeyWrapSymmetric, 2, objc._C_ID)
        self.assertArgHasType(
            Security.SecKeyUnwrapSymmetric, 3, objc._C_OUT + objc._C_PTR + objc._C_ID
        )

        self.assertResultHasType(Security.SecKeyGeneratePair, objc._C_INT)
        self.assertArgHasType(Security.SecKeyGeneratePair, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecKeyGeneratePair, 1, objc._C_OUT + objc._C_PTR + objc._C_ID
        )
        self.assertArgIsCFRetained(Security.SecKeyGeneratePair, 1)
        self.assertArgHasType(
            Security.SecKeyGeneratePair, 2, objc._C_OUT + objc._C_PTR + objc._C_ID
        )
        self.assertArgIsCFRetained(Security.SecKeyGeneratePair, 2)

        self.assertFalse(hasattr(Security, "SecKeyRawSign"))
        self.assertFalse(hasattr(Security, "SecKeyRawVerify"))
        self.assertFalse(hasattr(Security, "SecKeyEncrypt"))
        self.assertFalse(hasattr(Security, "SecKeyDecrypt"))

    @min_os_level("10.12")
    def test_functions_10_12(self):
        self.assertResultHasType(Security.SecKeyCreateRandomKey, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecKeyCreateRandomKey)
        self.assertArgHasType(Security.SecKeyCreateRandomKey, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecKeyCreateRandomKey, 1, objc._C_OUT + objc._C_PTR + objc._C_ID
        )

        self.assertResultHasType(Security.SecKeyCreateWithData, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecKeyCreateWithData)
        self.assertArgHasType(Security.SecKeyCreateWithData, 0, objc._C_ID)
        self.assertArgHasType(Security.SecKeyCreateWithData, 1, objc._C_ID)
        self.assertArgHasType(
            Security.SecKeyCreateWithData, 2, objc._C_OUT + objc._C_PTR + objc._C_ID
        )

        self.assertResultHasType(Security.SecKeyCopyExternalRepresentation, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecKeyCopyExternalRepresentation)
        self.assertArgHasType(Security.SecKeyCopyExternalRepresentation, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecKeyCopyExternalRepresentation,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )

        self.assertResultHasType(Security.SecKeyCopyAttributes, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecKeyCopyAttributes)
        self.assertArgHasType(Security.SecKeyCopyAttributes, 0, objc._C_ID)

        self.assertResultHasType(Security.SecKeyCopyPublicKey, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecKeyCopyPublicKey)
        self.assertArgHasType(Security.SecKeyCopyPublicKey, 0, objc._C_ID)

        self.assertResultHasType(Security.SecKeyCreateSignature, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecKeyCreateSignature)
        self.assertArgHasType(Security.SecKeyCreateSignature, 0, objc._C_ID)
        self.assertArgHasType(Security.SecKeyCreateSignature, 1, objc._C_ID)
        self.assertArgHasType(Security.SecKeyCreateSignature, 2, objc._C_ID)
        self.assertArgHasType(
            Security.SecKeyCreateSignature, 3, objc._C_OUT + objc._C_PTR + objc._C_ID
        )

        self.assertResultIsBOOL(Security.SecKeyVerifySignature)
        self.assertArgHasType(Security.SecKeyVerifySignature, 0, objc._C_ID)
        self.assertArgHasType(Security.SecKeyVerifySignature, 1, objc._C_ID)
        self.assertArgHasType(Security.SecKeyVerifySignature, 2, objc._C_ID)
        self.assertArgHasType(Security.SecKeyVerifySignature, 3, objc._C_ID)
        self.assertArgHasType(
            Security.SecKeyVerifySignature, 4, objc._C_OUT + objc._C_PTR + objc._C_ID
        )

        self.assertResultHasType(Security.SecKeyCreateEncryptedData, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecKeyCreateEncryptedData)
        self.assertArgHasType(Security.SecKeyCreateEncryptedData, 0, objc._C_ID)
        self.assertArgHasType(Security.SecKeyCreateEncryptedData, 1, objc._C_ID)
        self.assertArgHasType(Security.SecKeyCreateEncryptedData, 2, objc._C_ID)
        self.assertArgHasType(
            Security.SecKeyCreateEncryptedData,
            3,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )

        self.assertResultHasType(Security.SecKeyCreateDecryptedData, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecKeyCreateDecryptedData)
        self.assertArgHasType(Security.SecKeyCreateDecryptedData, 0, objc._C_ID)
        self.assertArgHasType(Security.SecKeyCreateDecryptedData, 1, objc._C_ID)
        self.assertArgHasType(Security.SecKeyCreateDecryptedData, 2, objc._C_ID)
        self.assertArgHasType(
            Security.SecKeyCreateDecryptedData,
            3,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )

        self.assertResultHasType(Security.SecKeyCopyKeyExchangeResult, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecKeyCopyKeyExchangeResult)
        self.assertArgHasType(Security.SecKeyCopyKeyExchangeResult, 0, objc._C_ID)
        self.assertArgHasType(Security.SecKeyCopyKeyExchangeResult, 1, objc._C_ID)
        self.assertArgHasType(Security.SecKeyCopyKeyExchangeResult, 2, objc._C_ID)
        self.assertArgHasType(Security.SecKeyCopyKeyExchangeResult, 3, objc._C_ID)
        self.assertArgHasType(
            Security.SecKeyCopyKeyExchangeResult,
            4,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )

        self.assertResultIsBOOL(Security.SecKeyIsAlgorithmSupported)
        self.assertArgHasType(Security.SecKeyIsAlgorithmSupported, 0, objc._C_ID)
        self.assertArgHasType(Security.SecKeyIsAlgorithmSupported, 1, objc._C_NSInteger)
        self.assertArgHasType(Security.SecKeyIsAlgorithmSupported, 2, objc._C_ID)

    def test_functions_deprecated(self):
        self.assertFalse(hasattr(Security, "SecKeyCreatePair"))
        self.assertFalse(hasattr(Security, "SecKeyGenerate"))
        self.assertFalse(hasattr(Security, "SecKeyGetCSSMKey"))
        self.assertFalse(hasattr(Security, "SecKeyGetCSPHandle"))
        self.assertFalse(hasattr(Security, "SecKeyGetCredentials"))



import Security
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestSecTransformReadTransform(TestCase):
    @min_os_level("10.7")
    def test_functions10_7(self):
        self.assertResultHasType(
            Security.SecTransformCreateReadTransformWithReadStream, objc._C_ID
        )
        self.assertResultIsCFRetained(
            Security.SecTransformCreateReadTransformWithReadStream
        )
        self.assertArgHasType(
            Security.SecTransformCreateReadTransformWithReadStream, 0, objc._C_ID
        )



import Security
from PyObjCTools.TestSupport import TestCase
import objc


class TestSecTrustSettings(TestCase):
    def test_constants(self):
        self.assertEqual(Security.kSecTrustSettingsPolicy, "kSecTrustSettingsPolicy")
        self.assertEqual(
            Security.kSecTrustSettingsApplication, "kSecTrustSettingsApplication"
        )
        self.assertEqual(
            Security.kSecTrustSettingsPolicyString, "kSecTrustSettingsPolicyString"
        )
        self.assertEqual(
            Security.kSecTrustSettingsKeyUsage, "kSecTrustSettingsKeyUsage"
        )
        self.assertEqual(
            Security.kSecTrustSettingsAllowedError, "kSecTrustSettingsAllowedError"
        )
        self.assertEqual(Security.kSecTrustSettingsResult, "kSecTrustSettingsResult")

        self.assertEqual(Security.kSecTrustSettingsKeyUseSignature, 0x00000001)
        self.assertEqual(Security.kSecTrustSettingsKeyUseEnDecryptData, 0x00000002)
        self.assertEqual(Security.kSecTrustSettingsKeyUseEnDecryptKey, 0x00000004)
        self.assertEqual(Security.kSecTrustSettingsKeyUseSignCert, 0x00000008)
        self.assertEqual(Security.kSecTrustSettingsKeyUseSignRevocation, 0x00000010)
        self.assertEqual(Security.kSecTrustSettingsKeyUseKeyExchange, 0x00000020)
        self.assertEqual(Security.kSecTrustSettingsKeyUseAny, 0xFFFFFFFF)

        self.assertEqual(Security.kSecTrustSettingsResultInvalid, 0)
        self.assertEqual(Security.kSecTrustSettingsResultTrustRoot, 1)
        self.assertEqual(Security.kSecTrustSettingsResultTrustAsRoot, 2)
        self.assertEqual(Security.kSecTrustSettingsResultDeny, 3)
        self.assertEqual(Security.kSecTrustSettingsResultUnspecified, 4)

        self.assertEqual(Security.kSecTrustSettingsDomainUser, 0)
        self.assertEqual(Security.kSecTrustSettingsDomainAdmin, 1)
        self.assertEqual(Security.kSecTrustSettingsDomainSystem, 2)

        self.assertEqual(Security.kSecTrustSettingsDefaultRootCertSetting, -1)

    def test_functions(self):
        self.assertResultHasType(
            Security.SecTrustSettingsCopyTrustSettings, objc._C_INT
        )
        self.assertArgHasType(Security.SecTrustSettingsCopyTrustSettings, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecTrustSettingsCopyTrustSettings, 1, objc._C_UINT
        )
        self.assertArgHasType(
            Security.SecTrustSettingsCopyTrustSettings,
            2,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )
        self.assertArgIsCFRetained(Security.SecTrustSettingsCopyTrustSettings, 2)

        self.assertResultHasType(Security.SecTrustSettingsSetTrustSettings, objc._C_INT)
        self.assertArgHasType(Security.SecTrustSettingsSetTrustSettings, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecTrustSettingsSetTrustSettings, 1, objc._C_UINT
        )
        self.assertArgHasType(Security.SecTrustSettingsSetTrustSettings, 2, objc._C_ID)

        self.assertResultHasType(
            Security.SecTrustSettingsRemoveTrustSettings, objc._C_INT
        )
        self.assertArgHasType(
            Security.SecTrustSettingsRemoveTrustSettings, 0, objc._C_ID
        )
        self.assertArgHasType(
            Security.SecTrustSettingsRemoveTrustSettings, 1, objc._C_UINT
        )

        self.assertResultHasType(Security.SecTrustSettingsCopyCertificates, objc._C_INT)
        self.assertArgHasType(
            Security.SecTrustSettingsCopyCertificates, 0, objc._C_UINT
        )
        self.assertArgHasType(
            Security.SecTrustSettingsCopyCertificates,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )
        self.assertArgIsCFRetained(Security.SecTrustSettingsCopyCertificates, 1)

        self.assertResultHasType(
            Security.SecTrustSettingsCopyModificationDate, objc._C_INT
        )
        self.assertArgHasType(
            Security.SecTrustSettingsCopyModificationDate, 0, objc._C_ID
        )
        self.assertArgHasType(
            Security.SecTrustSettingsCopyModificationDate, 1, objc._C_UINT
        )
        self.assertArgHasType(
            Security.SecTrustSettingsCopyModificationDate,
            2,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )
        self.assertArgIsCFRetained(Security.SecTrustSettingsCopyModificationDate, 2)

        self.assertResultHasType(
            Security.SecTrustSettingsCreateExternalRepresentation, objc._C_INT
        )
        self.assertArgHasType(
            Security.SecTrustSettingsCreateExternalRepresentation, 0, objc._C_UINT
        )
        self.assertArgHasType(
            Security.SecTrustSettingsCreateExternalRepresentation,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )
        self.assertArgIsCFRetained(
            Security.SecTrustSettingsCreateExternalRepresentation, 1
        )

        self.assertResultHasType(
            Security.SecTrustSettingsImportExternalRepresentation, objc._C_INT
        )
        self.assertArgHasType(
            Security.SecTrustSettingsImportExternalRepresentation, 0, objc._C_UINT
        )
        self.assertArgHasType(
            Security.SecTrustSettingsImportExternalRepresentation, 1, objc._C_ID
        )



import Security
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestSecCertificate(TestCase):
    def test_types(self):
        self.assertIsCFType(Security.SecCertificateRef)

    def test_constants(self):
        self.assertEqual(Security.kSecKeyUsageUnspecified, 0)
        self.assertEqual(Security.kSecKeyUsageDigitalSignature, 1 << 0)
        self.assertEqual(Security.kSecKeyUsageNonRepudiation, 1 << 1)
        self.assertEqual(Security.kSecKeyUsageContentCommitment, 1 << 1)
        self.assertEqual(Security.kSecKeyUsageKeyEncipherment, 1 << 2)
        self.assertEqual(Security.kSecKeyUsageDataEncipherment, 1 << 3)
        self.assertEqual(Security.kSecKeyUsageKeyAgreement, 1 << 4)
        self.assertEqual(Security.kSecKeyUsageKeyCertSign, 1 << 5)
        self.assertEqual(Security.kSecKeyUsageCRLSign, 1 << 6)
        self.assertEqual(Security.kSecKeyUsageEncipherOnly, 1 << 7)
        self.assertEqual(Security.kSecKeyUsageDecipherOnly, 1 << 8)
        self.assertEqual(Security.kSecKeyUsageCritical, 1 << 31)
        self.assertEqual(Security.kSecKeyUsageAll, 0x7FFFFFFF)

        self.assertIsInstance(Security.kSecPropertyKeyType, str)
        self.assertIsInstance(Security.kSecPropertyKeyLabel, str)
        self.assertIsInstance(Security.kSecPropertyKeyLocalizedLabel, str)
        self.assertIsInstance(Security.kSecPropertyKeyValue, str)

        self.assertIsInstance(Security.kSecPropertyTypeWarning, str)
        self.assertIsInstance(Security.kSecPropertyTypeSuccess, str)
        self.assertIsInstance(Security.kSecPropertyTypeSection, str)
        self.assertIsInstance(Security.kSecPropertyTypeData, str)
        self.assertIsInstance(Security.kSecPropertyTypeString, str)
        self.assertIsInstance(Security.kSecPropertyTypeURL, str)
        self.assertIsInstance(Security.kSecPropertyTypeDate, str)

        self.assertFalse(hasattr(Security, "kSecSubjectItemAttr"))
        self.assertFalse(hasattr(Security, "kSecIssuerItemAttr"))
        self.assertFalse(hasattr(Security, "kSecSerialNumberItemAttr"))
        self.assertFalse(hasattr(Security, "kSecPublicKeyHashItemAttr"))
        self.assertFalse(hasattr(Security, "kSecSubjectKeyIdentifierItemAttr"))
        self.assertFalse(hasattr(Security, "kSecCertTypeItemAttr"))
        self.assertFalse(hasattr(Security, "kSecCertEncodingItemAttr"))

    @min_os_level("10.15")
    def test_constants10_15(self):
        self.assertIsInstance(Security.kSecPropertyTypeArray, str)
        self.assertIsInstance(Security.kSecPropertyTypeNumber, str)

    def test_functions(self):
        self.assertIsInstance(Security.SecCertificateGetTypeID(), int)

        self.assertResultHasType(Security.SecCertificateCreateWithData, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecCertificateCreateWithData)
        self.assertArgHasType(Security.SecCertificateCreateWithData, 0, objc._C_ID)
        self.assertArgHasType(Security.SecCertificateCreateWithData, 1, objc._C_ID)

        self.assertResultHasType(Security.SecCertificateCopyData, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecCertificateCopyData)
        self.assertArgHasType(Security.SecCertificateCopyData, 0, objc._C_ID)

        self.assertResultHasType(Security.SecCertificateCopySubjectSummary, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecCertificateCopySubjectSummary)
        self.assertArgHasType(Security.SecCertificateCopySubjectSummary, 0, objc._C_ID)

        self.assertResultHasType(Security.SecCertificateCopyCommonName, objc._C_INT)
        self.assertArgHasType(Security.SecCertificateCopyCommonName, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecCertificateCopyCommonName,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )
        self.assertArgIsCFRetained(Security.SecCertificateCopyCommonName, 1)

        self.assertResultHasType(Security.SecCertificateCopyEmailAddresses, objc._C_INT)
        self.assertArgHasType(Security.SecCertificateCopyEmailAddresses, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecCertificateCopyEmailAddresses,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )
        self.assertArgIsCFRetained(Security.SecCertificateCopyEmailAddresses, 1)

        # "SEC_SUFFIX_LEGACYMAC" on arm64
        self.assertResultHasType(Security.SecCertificateCopyPublicKey, objc._C_INT)
        self.assertArgHasType(Security.SecCertificateCopyPublicKey, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecCertificateCopyPublicKey,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )
        self.assertArgIsCFRetained(Security.SecCertificateCopyPublicKey, 1)

        # "SEC_SUFFIX_LEGACYMAC" on arm64
        self.assertResultHasType(Security.SecCertificateCopySerialNumber, objc._C_ID)
        self.assertArgHasType(Security.SecCertificateCopySerialNumber, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecCertificateCopySerialNumber,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )

        self.assertResultHasType(Security.SecCertificateAddToKeychain, objc._C_INT)
        self.assertArgHasType(Security.SecCertificateAddToKeychain, 0, objc._C_ID)
        self.assertArgHasType(Security.SecCertificateAddToKeychain, 1, objc._C_ID)

        self.assertResultHasType(Security.SecCertificateCopyPreferred, objc._C_ID)
        self.assertArgHasType(Security.SecCertificateCopyPreferred, 0, objc._C_ID)
        self.assertArgHasType(Security.SecCertificateCopyPreferred, 1, objc._C_ID)

        self.assertResultHasType(Security.SecCertificateSetPreferred, objc._C_INT)
        self.assertArgHasType(Security.SecCertificateSetPreferred, 0, objc._C_ID)
        self.assertArgHasType(Security.SecCertificateSetPreferred, 1, objc._C_ID)
        self.assertArgHasType(Security.SecCertificateSetPreferred, 2, objc._C_ID)

        self.assertResultHasType(Security.SecCertificateCopyValues, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecCertificateCopyValues)
        self.assertArgHasType(Security.SecCertificateCopyValues, 0, objc._C_ID)
        self.assertArgHasType(Security.SecCertificateCopyValues, 1, objc._C_ID)
        self.assertArgHasType(
            Security.SecCertificateCopyValues, 2, objc._C_OUT + objc._C_PTR + objc._C_ID
        )

        self.assertResultHasType(Security.SecCertificateCopyLongDescription, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecCertificateCopyLongDescription)
        self.assertArgHasType(Security.SecCertificateCopyLongDescription, 0, objc._C_ID)
        self.assertArgHasType(Security.SecCertificateCopyLongDescription, 1, objc._C_ID)
        self.assertArgHasType(
            Security.SecCertificateCopyLongDescription,
            2,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )

        self.assertResultHasType(
            Security.SecCertificateCopyShortDescription, objc._C_ID
        )
        self.assertResultIsCFRetained(Security.SecCertificateCopyShortDescription)
        self.assertArgHasType(
            Security.SecCertificateCopyShortDescription, 0, objc._C_ID
        )
        self.assertArgHasType(
            Security.SecCertificateCopyShortDescription, 1, objc._C_ID
        )
        self.assertArgHasType(
            Security.SecCertificateCopyShortDescription,
            2,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )

        self.assertResultHasType(
            Security.SecCertificateCopyNormalizedIssuerContent, objc._C_ID
        )
        self.assertResultIsCFRetained(
            Security.SecCertificateCopyNormalizedIssuerContent
        )
        self.assertArgHasType(
            Security.SecCertificateCopyNormalizedIssuerContent, 0, objc._C_ID
        )
        self.assertArgHasType(
            Security.SecCertificateCopyNormalizedIssuerContent,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )

        self.assertResultHasType(
            Security.SecCertificateCopyNormalizedSubjectContent, objc._C_ID
        )
        self.assertResultIsCFRetained(
            Security.SecCertificateCopyNormalizedSubjectContent
        )
        self.assertArgHasType(
            Security.SecCertificateCopyNormalizedSubjectContent, 0, objc._C_ID
        )
        self.assertArgHasType(
            Security.SecCertificateCopyNormalizedSubjectContent,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )

        self.assertFalse(hasattr(Security, "SecCertificateCreateFromData"))
        self.assertFalse(hasattr(Security, "SecCertificateGetData"))
        self.assertFalse(hasattr(Security, "SecCertificateGetType"))
        self.assertFalse(hasattr(Security, "SecCertificateGetSubject"))
        self.assertFalse(hasattr(Security, "SecCertificateGetIssuer"))
        self.assertFalse(hasattr(Security, "SecCertificateGetCLHandle"))
        self.assertFalse(hasattr(Security, "SecCertificateGetAlgorithmID"))
        self.assertFalse(hasattr(Security, "SecCertificateCopyPreference"))
        self.assertFalse(hasattr(Security, "SecCertificateSetPreference"))

    @min_os_level("10.12.4")
    def test_functions_10_12_4(self):
        self.assertResultHasType(
            Security.SecCertificateCopyNormalizedIssuerSequence, objc._C_ID
        )
        self.assertResultIsCFRetained(
            Security.SecCertificateCopyNormalizedIssuerSequence
        )
        self.assertArgHasType(
            Security.SecCertificateCopyNormalizedIssuerSequence, 0, objc._C_ID
        )

        self.assertResultHasType(
            Security.SecCertificateCopyNormalizedSubjectSequence, objc._C_ID
        )
        self.assertResultIsCFRetained(
            Security.SecCertificateCopyNormalizedSubjectSequence
        )
        self.assertArgHasType(
            Security.SecCertificateCopyNormalizedSubjectSequence, 0, objc._C_ID
        )

    @min_os_level("10.13")
    def test_functions_10_13(self):
        self.assertResultHasType(
            Security.SecCertificateCopySerialNumberData, objc._C_ID
        )
        self.assertResultIsCFRetained(Security.SecCertificateCopySerialNumberData)
        self.assertArgHasType(
            Security.SecCertificateCopySerialNumberData, 0, objc._C_ID
        )
        self.assertArgHasType(
            Security.SecCertificateCopySerialNumberData,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )

    @min_os_level("10.14")
    def test_functions_10_14(self):
        self.assertResultHasType(Security.SecCertificateCopyKey, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecCertificateCopyKey)
        self.assertArgHasType(Security.SecCertificateCopyKey, 0, objc._C_ID)



import Security
from PyObjCTools.TestSupport import TestCase, min_os_level, expectedFailure
import objc

SSLReadFunc = b"i@o^vN^L"
SSLWriteFunc = b"i@n^vN^L"


class TestSecureTransport(TestCase):
    def test_types(self):
        self.assertIsCFType(Security.SSLContextRef)

    def test_constants(self):
        self.assertEqual(Security.kSSLCiphersuiteGroupDefault, 0)
        self.assertEqual(Security.kSSLCiphersuiteGroupCompatibility, 1)
        self.assertEqual(Security.kSSLCiphersuiteGroupLegacy, 2)
        self.assertEqual(Security.kSSLCiphersuiteGroupATS, 3)
        self.assertEqual(Security.kSSLCiphersuiteGroupATSCompatibility, 4)

        self.assertEqual(Security.kSSLProtocolUnknown, 0)
        self.assertEqual(Security.kSSLProtocol3, 2)
        self.assertEqual(Security.kTLSProtocol1, 4)
        self.assertEqual(Security.kTLSProtocol11, 7)
        self.assertEqual(Security.kTLSProtocol12, 8)
        self.assertEqual(Security.kDTLSProtocol1, 9)
        self.assertEqual(Security.kTLSProtocol13, 10)
        self.assertEqual(Security.kTLSProtocolMaxSupported, 999)
        self.assertEqual(Security.kSSLProtocol2, 1)
        self.assertEqual(Security.kSSLProtocol3Only, 3)
        self.assertEqual(Security.kTLSProtocol1Only, 5)
        self.assertEqual(Security.kSSLProtocolAll, 6)

        self.assertEqual(Security.kSSLSessionOptionBreakOnServerAuth, 0)
        self.assertEqual(Security.kSSLSessionOptionBreakOnCertRequested, 1)
        self.assertEqual(Security.kSSLSessionOptionBreakOnClientAuth, 2)
        self.assertEqual(Security.kSSLSessionOptionFalseStart, 3)
        self.assertEqual(Security.kSSLSessionOptionSendOneByteRecord, 4)
        self.assertEqual(Security.kSSLSessionOptionAllowServerIdentityChange, 5)
        self.assertEqual(Security.kSSLSessionOptionFallback, 6)
        self.assertEqual(Security.kSSLSessionOptionBreakOnClientHello, 7)
        self.assertEqual(Security.kSSLSessionOptionAllowRenegotiation, 8)
        self.assertEqual(Security.kSSLSessionOptionEnableSessionTickets, 9)

        self.assertEqual(Security.kSSLIdle, 0)
        self.assertEqual(Security.kSSLHandshake, 1)
        self.assertEqual(Security.kSSLConnected, 2)
        self.assertEqual(Security.kSSLClosed, 3)
        self.assertEqual(Security.kSSLAborted, 4)

        self.assertEqual(Security.kSSLClientCertNone, 0)
        self.assertEqual(Security.kSSLClientCertRequested, 1)
        self.assertEqual(Security.kSSLClientCertSent, 2)
        self.assertEqual(Security.kSSLClientCertRejected, 3)

        self.assertEqual(Security.errSSLProtocol, -9800)
        self.assertEqual(Security.errSSLNegotiation, -9801)
        self.assertEqual(Security.errSSLFatalAlert, -9802)
        self.assertEqual(Security.errSSLWouldBlock, -9803)
        self.assertEqual(Security.errSSLSessionNotFound, -9804)
        self.assertEqual(Security.errSSLClosedGraceful, -9805)
        self.assertEqual(Security.errSSLClosedAbort, -9806)
        self.assertEqual(Security.errSSLXCertChainInvalid, -9807)
        self.assertEqual(Security.errSSLBadCert, -9808)
        self.assertEqual(Security.errSSLCrypto, -9809)
        self.assertEqual(Security.errSSLInternal, -9810)
        self.assertEqual(Security.errSSLModuleAttach, -9811)
        self.assertEqual(Security.errSSLUnknownRootCert, -9812)
        self.assertEqual(Security.errSSLNoRootCert, -9813)
        self.assertEqual(Security.errSSLCertExpired, -9814)
        self.assertEqual(Security.errSSLCertNotYetValid, -9815)
        self.assertEqual(Security.errSSLClosedNoNotify, -9816)
        self.assertEqual(Security.errSSLBufferOverflow, -9817)
        self.assertEqual(Security.errSSLBadCipherSuite, -9818)
        self.assertEqual(Security.errSSLPeerUnexpectedMsg, -9819)
        self.assertEqual(Security.errSSLPeerBadRecordMac, -9820)
        self.assertEqual(Security.errSSLPeerDecryptionFail, -9821)
        self.assertEqual(Security.errSSLPeerRecordOverflow, -9822)
        self.assertEqual(Security.errSSLPeerDecompressFail, -9823)
        self.assertEqual(Security.errSSLPeerHandshakeFail, -9824)
        self.assertEqual(Security.errSSLPeerBadCert, -9825)
        self.assertEqual(Security.errSSLPeerUnsupportedCert, -9826)
        self.assertEqual(Security.errSSLPeerCertRevoked, -9827)
        self.assertEqual(Security.errSSLPeerCertExpired, -9828)
        self.assertEqual(Security.errSSLPeerCertUnknown, -9829)
        self.assertEqual(Security.errSSLIllegalParam, -9830)
        self.assertEqual(Security.errSSLPeerUnknownCA, -9831)
        self.assertEqual(Security.errSSLPeerAccessDenied, -9832)
        self.assertEqual(Security.errSSLPeerDecodeError, -9833)
        self.assertEqual(Security.errSSLPeerDecryptError, -9834)
        self.assertEqual(Security.errSSLPeerExportRestriction, -9835)
        self.assertEqual(Security.errSSLPeerProtocolVersion, -9836)
        self.assertEqual(Security.errSSLPeerInsufficientSecurity, -9837)
        self.assertEqual(Security.errSSLPeerInternalError, -9838)
        self.assertEqual(Security.errSSLPeerUserCancelled, -9839)
        self.assertEqual(Security.errSSLPeerNoRenegotiation, -9840)
        self.assertEqual(Security.errSSLPeerAuthCompleted, -9841)
        self.assertEqual(Security.errSSLClientCertRequested, -9842)
        self.assertEqual(Security.errSSLHostNameMismatch, -9843)
        self.assertEqual(Security.errSSLConnectionRefused, -9844)
        self.assertEqual(Security.errSSLDecryptionFail, -9845)
        self.assertEqual(Security.errSSLBadRecordMac, -9846)
        self.assertEqual(Security.errSSLRecordOverflow, -9847)
        self.assertEqual(Security.errSSLBadConfiguration, -9848)
        self.assertEqual(Security.errSSLUnexpectedRecord, -9849)
        self.assertEqual(Security.errSSLWeakPeerEphemeralDHKey, -9850)
        self.assertEqual(Security.errSSLClientHelloReceived, -9851)

        self.assertEqual(
            Security.errSSLServerAuthCompleted, Security.errSSLPeerAuthCompleted
        )
        self.assertEqual(
            Security.errSSLClientAuthCompleted, Security.errSSLPeerAuthCompleted
        )
        self.assertEqual(Security.errSSLLast, Security.errSSLUnexpectedRecord)

        self.assertEqual(Security.kSSLServerSide, 0)
        self.assertEqual(Security.kSSLClientSide, 1)

        self.assertEqual(Security.kSSLStreamType, 0)
        self.assertEqual(Security.kSSLDatagramType, 1)

        self.assertEqual(Security.kNeverAuthenticate, 0)
        self.assertEqual(Security.kAlwaysAuthenticate, 1)
        self.assertEqual(Security.kTryAuthenticate, 2)

        self.assertEqual(Security.errSSLTransportReset, -9852)
        self.assertEqual(Security.errSSLNetworkTimeout, -9853)
        self.assertEqual(Security.errSSLConfigurationFailed, -9854)
        self.assertEqual(Security.errSSLUnsupportedExtension, -9855)
        self.assertEqual(Security.errSSLUnexpectedMessage, -9856)
        self.assertEqual(Security.errSSLDecompressFail, -9857)
        self.assertEqual(Security.errSSLHandshakeFail, -9858)
        self.assertEqual(Security.errSSLDecodeError, -9859)
        self.assertEqual(Security.errSSLInappropriateFallback, -9860)
        self.assertEqual(Security.errSSLMissingExtension, -9861)
        self.assertEqual(Security.errSSLBadCertificateStatusResponse, -9862)
        self.assertEqual(Security.errSSLCertificateRequired, -9863)
        self.assertEqual(Security.errSSLUnknownPSKIdentity, -9864)
        self.assertEqual(Security.errSSLUnrecognizedName, -9865)

    @min_os_level("10.12")
    def test_constants_10_12(self):
        self.assertIsInstance(Security.kSSLSessionConfig_default, str)
        self.assertIsInstance(Security.kSSLSessionConfig_ATSv1, str)
        self.assertIsInstance(Security.kSSLSessionConfig_ATSv1_noPFS, str)
        self.assertIsInstance(Security.kSSLSessionConfig_standard, str)
        self.assertIsInstance(Security.kSSLSessionConfig_RC4_fallback, str)
        self.assertIsInstance(Security.kSSLSessionConfig_TLSv1_fallback, str)
        self.assertIsInstance(Security.kSSLSessionConfig_TLSv1_RC4_fallback, str)
        self.assertIsInstance(Security.kSSLSessionConfig_legacy, str)
        self.assertIsInstance(Security.kSSLSessionConfig_legacy_DHE, str)
        self.assertIsInstance(Security.kSSLSessionConfig_anonymous, str)
        self.assertIsInstance(Security.kSSLSessionConfig_3DES_fallback, str)
        self.assertIsInstance(Security.kSSLSessionConfig_TLSv1_3DES_fallback, str)

    def test_functions(self):
        self.assertIsInstance(Security.SSLContextGetTypeID(), int)

        self.assertFalse(hasattr(Security, "SSLNewContext"))
        self.assertFalse(hasattr(Security, "SSLDisposeContext"))

        self.assertResultHasType(Security.SSLGetSessionState, objc._C_INT)
        self.assertArgHasType(Security.SSLGetSessionState, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SSLGetSessionState, 1, objc._C_OUT + objc._C_PTR + objc._C_INT
        )

        self.assertResultHasType(Security.SSLSetIOFuncs, objc._C_INT)
        self.assertArgHasType(Security.SSLSetIOFuncs, 0, objc._C_ID)
        self.assertArgIsFunction(Security.SSLSetIOFuncs, 1, SSLReadFunc, True)
        self.assertArgIsFunction(Security.SSLSetIOFuncs, 2, SSLWriteFunc, True)

        self.assertResultHasType(Security.SSLSetCertificate, objc._C_INT)
        self.assertArgHasType(Security.SSLSetCertificate, 0, objc._C_ID)
        self.assertArgHasType(Security.SSLSetCertificate, 1, objc._C_ID)

        self.assertResultHasType(Security.SSLSetConnection, objc._C_INT)
        self.assertArgHasType(Security.SSLSetConnection, 0, objc._C_ID)
        self.assertArgHasType(Security.SSLSetConnection, 1, objc._C_ID)

        self.assertResultHasType(Security.SSLGetConnection, objc._C_INT)
        self.assertArgHasType(Security.SSLGetConnection, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SSLGetConnection, 1, objc._C_OUT + objc._C_PTR + objc._C_ID
        )

        self.assertResultHasType(Security.SSLSetPeerDomainName, objc._C_INT)
        self.assertArgHasType(Security.SSLSetPeerDomainName, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SSLSetPeerDomainName,
            1,
            objc._C_IN + objc._C_PTR + objc._C_CHAR_AS_TEXT,
        )
        self.assertArgSizeInArg(Security.SSLSetPeerDomainName, 1, 2)
        self.assertArgHasType(Security.SSLSetPeerDomainName, 2, objc._C_ULNG)

        self.assertResultHasType(Security.SSLGetPeerDomainNameLength, objc._C_INT)
        self.assertArgHasType(Security.SSLGetPeerDomainNameLength, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SSLGetPeerDomainNameLength,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_ULNG,
        )

        self.assertResultHasType(Security.SSLGetPeerDomainName, objc._C_INT)
        self.assertArgHasType(Security.SSLGetPeerDomainName, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SSLGetPeerDomainName,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_CHAR_AS_TEXT,
        )
        self.assertArgSizeInArg(Security.SSLGetPeerDomainName, 1, 2)
        self.assertArgHasType(
            Security.SSLGetPeerDomainName, 2, objc._C_INOUT + objc._C_PTR + objc._C_ULNG
        )

        self.assertResultHasType(Security.SSLGetNegotiatedProtocolVersion, objc._C_INT)
        self.assertArgHasType(Security.SSLGetNegotiatedProtocolVersion, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SSLGetNegotiatedProtocolVersion,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_INT,
        )

        self.assertResultHasType(Security.SSLGetNumberSupportedCiphers, objc._C_INT)
        self.assertArgHasType(Security.SSLGetNumberSupportedCiphers, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SSLGetNumberSupportedCiphers,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_ULNG,
        )

        self.assertResultHasType(Security.SSLGetSupportedCiphers, objc._C_INT)
        self.assertArgHasType(Security.SSLGetSupportedCiphers, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SSLGetSupportedCiphers, 1, objc._C_OUT + objc._C_PTR + objc._C_INT
        )
        self.assertArgSizeInArg(Security.SSLGetSupportedCiphers, 1, 2)
        self.assertArgHasType(
            Security.SSLGetSupportedCiphers,
            2,
            objc._C_INOUT + objc._C_PTR + objc._C_ULNG,
        )

        self.assertResultHasType(Security.SSLSetEnabledCiphers, objc._C_INT)
        self.assertArgHasType(Security.SSLSetEnabledCiphers, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SSLSetEnabledCiphers, 1, objc._C_IN + objc._C_PTR + objc._C_INT
        )
        self.assertArgSizeInArg(Security.SSLSetEnabledCiphers, 1, 2)
        self.assertArgHasType(Security.SSLSetEnabledCiphers, 2, objc._C_ULNG)

        self.assertResultHasType(Security.SSLGetNumberEnabledCiphers, objc._C_INT)
        self.assertArgHasType(Security.SSLGetNumberEnabledCiphers, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SSLGetNumberEnabledCiphers,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_ULNG,
        )

        self.assertResultHasType(Security.SSLGetEnabledCiphers, objc._C_INT)
        self.assertArgHasType(Security.SSLGetEnabledCiphers, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SSLGetEnabledCiphers, 1, objc._C_OUT + objc._C_PTR + objc._C_INT
        )
        self.assertArgSizeInArg(Security.SSLGetEnabledCiphers, 1, 2)
        self.assertArgHasType(
            Security.SSLGetEnabledCiphers, 2, objc._C_INOUT + objc._C_PTR + objc._C_ULNG
        )

        self.assertFalse(hasattr(Security, "SSLSetProtocolVersionEnabled"))
        self.assertFalse(hasattr(Security, "SSLGetProtocolVersionEnabled"))
        self.assertFalse(hasattr(Security, "SSLSetProtocolVersion"))
        self.assertFalse(hasattr(Security, "SSLGetProtocolVersion"))
        self.assertFalse(hasattr(Security, "SSLSetEnableCertVerify"))
        self.assertFalse(hasattr(Security, "SSLGetEnableCertVerify"))
        self.assertFalse(hasattr(Security, "SSLSetAllowsExpiredCerts"))
        self.assertFalse(hasattr(Security, "SSLGetAllowsExpiredCerts"))
        self.assertFalse(hasattr(Security, "SSLSetAllowsExpiredRoots"))
        self.assertFalse(hasattr(Security, "SSLGetAllowsExpiredRoots"))
        self.assertFalse(hasattr(Security, "SSLSetAllowsAnyRoot"))
        self.assertFalse(hasattr(Security, "SSLGetAllowsAnyRoot"))
        self.assertFalse(hasattr(Security, "SSLSetTrustedRoots"))
        self.assertFalse(hasattr(Security, "SSLCopyTrustedRoots"))
        self.assertFalse(hasattr(Security, "SSLCopyPeerCertificates"))

        self.assertResultHasType(Security.SSLSetPeerID, objc._C_INT)
        self.assertArgHasType(Security.SSLSetPeerID, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SSLSetPeerID, 1, objc._C_IN + objc._C_PTR + objc._C_VOID
        )
        self.assertArgSizeInArg(Security.SSLSetPeerID, 1, 2)
        self.assertArgHasType(Security.SSLSetPeerID, 2, objc._C_ULNG)

        self.assertResultHasType(Security.SSLGetPeerID, objc._C_INT)
        self.assertArgHasType(Security.SSLGetPeerID, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SSLGetPeerID, 1, objc._C_OUT + objc._C_PTR + objc._C_VOID
        )
        self.assertArgSizeInArg(Security.SSLGetPeerID, 1, 2)
        self.assertArgHasType(
            Security.SSLGetPeerID, 2, objc._C_INOUT + objc._C_PTR + objc._C_ULNG
        )

        self.assertResultHasType(Security.SSLGetNegotiatedCipher, objc._C_INT)
        self.assertArgHasType(Security.SSLGetNegotiatedCipher, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SSLGetNegotiatedCipher, 1, objc._C_OUT + objc._C_PTR + objc._C_INT
        )

        self.assertResultHasType(Security.SSLSetClientSideAuthenticate, objc._C_INT)
        self.assertArgHasType(Security.SSLSetClientSideAuthenticate, 0, objc._C_ID)
        self.assertArgHasType(Security.SSLSetClientSideAuthenticate, 1, objc._C_INT)

        self.assertResultHasType(Security.SSLAddDistinguishedName, objc._C_INT)
        self.assertArgHasType(Security.SSLAddDistinguishedName, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SSLAddDistinguishedName, 1, objc._C_IN + objc._C_PTR + objc._C_VOID
        )
        self.assertArgSizeInArg(Security.SSLAddDistinguishedName, 1, 2)
        self.assertArgHasType(Security.SSLAddDistinguishedName, 2, objc._C_ULNG)

        self.assertResultHasType(Security.SSLSetCertificateAuthorities, objc._C_INT)
        self.assertArgHasType(Security.SSLSetCertificateAuthorities, 0, objc._C_ID)
        self.assertArgHasType(Security.SSLSetCertificateAuthorities, 1, objc._C_ID)
        self.assertArgHasType(Security.SSLSetCertificateAuthorities, 2, objc._C_NSBOOL)

        self.assertResultHasType(Security.SSLCopyCertificateAuthorities, objc._C_INT)
        self.assertArgHasType(Security.SSLCopyCertificateAuthorities, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SSLCopyCertificateAuthorities,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )
        self.assertArgIsCFRetained(Security.SSLCopyCertificateAuthorities, 1)

        self.assertResultHasType(Security.SSLCopyDistinguishedNames, objc._C_INT)
        self.assertArgHasType(Security.SSLCopyDistinguishedNames, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SSLCopyDistinguishedNames,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )
        self.assertArgIsCFRetained(Security.SSLCopyDistinguishedNames, 1)

        self.assertResultHasType(Security.SSLGetClientCertificateState, objc._C_INT)
        self.assertArgHasType(Security.SSLGetClientCertificateState, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SSLGetClientCertificateState,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_INT,
        )

        self.assertResultHasType(Security.SSLSetDiffieHellmanParams, objc._C_INT)
        self.assertArgHasType(Security.SSLSetDiffieHellmanParams, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SSLSetDiffieHellmanParams,
            1,
            objc._C_IN + objc._C_PTR + objc._C_VOID,
        )
        self.assertArgSizeInArg(Security.SSLSetDiffieHellmanParams, 1, 2)
        self.assertArgHasType(Security.SSLSetDiffieHellmanParams, 2, objc._C_ULNG)

        self.assertResultHasType(Security.SSLGetDiffieHellmanParams, objc._C_INT)
        self.assertArgHasType(Security.SSLGetDiffieHellmanParams, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SSLGetDiffieHellmanParams,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_VOID,
        )
        self.assertArgSizeInArg(Security.SSLGetDiffieHellmanParams, 1, 2)
        self.assertArgHasType(
            Security.SSLGetDiffieHellmanParams,
            2,
            objc._C_INOUT + objc._C_PTR + objc._C_ULNG,
        )

        self.assertFalse(hasattr(Security, "SSLSetRsaBlinding"))
        self.assertFalse(hasattr(Security, "SSLGetRsaBlinding"))

        self.assertResultHasType(Security.SSLHandshake, objc._C_INT)
        self.assertArgHasType(Security.SSLHandshake, 0, objc._C_ID)

        self.assertResultHasType(Security.SSLWrite, objc._C_INT)
        self.assertArgHasType(Security.SSLWrite, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SSLWrite, 1, objc._C_IN + objc._C_PTR + objc._C_VOID
        )
        self.assertArgSizeInArg(Security.SSLWrite, 1, 2)
        self.assertArgHasType(Security.SSLWrite, 2, objc._C_ULNG)
        self.assertArgHasType(
            Security.SSLWrite, 3, objc._C_OUT + objc._C_PTR + objc._C_ULNG
        )

        self.assertResultHasType(Security.SSLRead, objc._C_INT)
        self.assertArgHasType(Security.SSLRead, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SSLRead, 1, objc._C_OUT + objc._C_PTR + objc._C_VOID
        )
        self.assertArgSizeInArg(Security.SSLRead, 1, (2, 3))
        self.assertArgHasType(Security.SSLRead, 2, objc._C_ULNG)
        self.assertArgHasType(
            Security.SSLRead, 3, objc._C_OUT + objc._C_PTR + objc._C_ULNG
        )

        self.assertResultHasType(Security.SSLGetBufferedReadSize, objc._C_INT)
        self.assertArgHasType(Security.SSLGetBufferedReadSize, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SSLGetBufferedReadSize, 1, objc._C_OUT + objc._C_PTR + objc._C_ULNG
        )

        self.assertResultHasType(Security.SSLClose, objc._C_INT)
        self.assertArgHasType(Security.SSLClose, 0, objc._C_ID)

    @min_os_level("10.6")
    def test_functions10_6(self):
        self.assertResultHasType(Security.SSLSetSessionOption, objc._C_INT)
        self.assertArgHasType(Security.SSLSetSessionOption, 0, objc._C_ID)
        self.assertArgHasType(Security.SSLSetSessionOption, 1, objc._C_INT)
        self.assertArgHasType(Security.SSLSetSessionOption, 2, objc._C_NSBOOL)

        self.assertResultHasType(Security.SSLGetSessionOption, objc._C_INT)
        self.assertArgHasType(Security.SSLGetSessionOption, 0, objc._C_ID)
        self.assertArgHasType(Security.SSLGetSessionOption, 1, objc._C_INT)
        self.assertArgHasType(
            Security.SSLGetSessionOption, 2, objc._C_OUT + objc._C_PTR + objc._C_NSBOOL
        )

        self.assertResultHasType(Security.SSLCopyPeerTrust, objc._C_INT)
        self.assertArgHasType(Security.SSLCopyPeerTrust, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SSLCopyPeerTrust, 1, objc._C_OUT + objc._C_PTR + objc._C_ID
        )
        self.assertArgIsCFRetained(Security.SSLCopyPeerTrust, 1)

    @min_os_level("10.8")
    def test_functions10_8(self):
        self.assertResultHasType(Security.SSLCreateContext, objc._C_ID)
        self.assertResultIsCFRetained(Security.SSLCreateContext)
        self.assertArgHasType(Security.SSLCreateContext, 0, objc._C_ID)
        self.assertArgHasType(Security.SSLCreateContext, 1, objc._C_INT)
        self.assertArgHasType(Security.SSLCreateContext, 2, objc._C_INT)

        self.assertResultHasType(Security.SSLSetProtocolVersionMin, objc._C_INT)
        self.assertArgHasType(Security.SSLSetProtocolVersionMin, 0, objc._C_ID)
        self.assertArgHasType(Security.SSLSetProtocolVersionMin, 1, objc._C_INT)

        self.assertResultHasType(Security.SSLGetProtocolVersionMin, objc._C_INT)
        self.assertArgHasType(Security.SSLGetProtocolVersionMin, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SSLGetProtocolVersionMin,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_INT,
        )

        self.assertResultHasType(Security.SSLSetProtocolVersionMax, objc._C_INT)
        self.assertArgHasType(Security.SSLSetProtocolVersionMax, 0, objc._C_ID)
        self.assertArgHasType(Security.SSLSetProtocolVersionMax, 1, objc._C_INT)

        self.assertResultHasType(Security.SSLGetProtocolVersionMax, objc._C_INT)
        self.assertArgHasType(Security.SSLGetProtocolVersionMax, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SSLGetProtocolVersionMax,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_INT,
        )

        self.assertResultHasType(Security.SSLSetDatagramHelloCookie, objc._C_INT)
        self.assertArgHasType(Security.SSLSetDatagramHelloCookie, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SSLSetDatagramHelloCookie,
            1,
            objc._C_IN + objc._C_PTR + objc._C_VOID,
        )
        self.assertArgSizeInArg(Security.SSLSetDatagramHelloCookie, 1, 2)
        self.assertArgHasType(Security.SSLSetDatagramHelloCookie, 2, objc._C_ULNG)

        self.assertResultHasType(Security.SSLSetMaxDatagramRecordSize, objc._C_INT)
        self.assertArgHasType(Security.SSLSetMaxDatagramRecordSize, 0, objc._C_ID)
        self.assertArgHasType(Security.SSLSetMaxDatagramRecordSize, 1, objc._C_ULNG)

        self.assertResultHasType(Security.SSLGetMaxDatagramRecordSize, objc._C_INT)
        self.assertArgHasType(Security.SSLGetMaxDatagramRecordSize, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SSLGetMaxDatagramRecordSize,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_ULNG,
        )

        self.assertResultHasType(Security.SSLGetDatagramWriteSize, objc._C_INT)
        self.assertArgHasType(Security.SSLGetDatagramWriteSize, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SSLGetDatagramWriteSize,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_ULNG,
        )

    @min_os_level("10.11")
    def test_functions_10_11(self):
        self.assertResultHasType(Security.SSLCopyRequestedPeerName, objc._C_INT)
        self.assertArgHasType(Security.SSLCopyRequestedPeerName, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SSLCopyRequestedPeerName,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_CHAR_AS_TEXT,
        )
        self.assertArgSizeInArg(Security.SSLCopyRequestedPeerName, 1, 2)
        self.assertArgHasType(
            Security.SSLGetPeerDomainName, 2, objc._C_INOUT + objc._C_PTR + objc._C_ULNG
        )

        self.assertResultHasType(Security.SSLCopyRequestedPeerNameLength, objc._C_INT)
        self.assertArgHasType(Security.SSLCopyRequestedPeerNameLength, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SSLCopyRequestedPeerNameLength,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_ULNG,
        )

    @min_os_level("10.12")
    def test_functions10_12(self):
        self.assertResultHasType(Security.SSLSetSessionConfig, objc._C_INT)
        self.assertArgHasType(Security.SSLSetSessionConfig, 0, objc._C_ID)
        self.assertArgHasType(Security.SSLSetSessionConfig, 1, objc._C_ID)

        self.assertResultHasType(Security.SSLReHandshake, objc._C_INT)
        self.assertArgHasType(Security.SSLReHandshake, 0, objc._C_ID)

    @min_os_level("10.13")
    def test_functions10_13(self):
        self.assertResultHasType(Security.SSLSetSessionTicketsEnabled, objc._C_INT)
        self.assertArgHasType(Security.SSLSetSessionTicketsEnabled, 0, objc._C_ID)
        self.assertArgHasType(Security.SSLSetSessionTicketsEnabled, 1, objc._C_NSBOOL)

        self.assertResultHasType(Security.SSLSetOCSPResponse, objc._C_INT)
        self.assertArgHasType(Security.SSLSetOCSPResponse, 0, objc._C_ID)
        self.assertArgHasType(Security.SSLSetOCSPResponse, 1, objc._C_ID)

        self.assertResultHasType(Security.SSLSetEncryptionCertificate, objc._C_INT)
        self.assertArgHasType(Security.SSLSetEncryptionCertificate, 0, objc._C_ID)
        self.assertArgHasType(Security.SSLSetEncryptionCertificate, 1, objc._C_ID)

        self.assertResultHasType(Security.SSLSetError, objc._C_INT)
        self.assertArgHasType(Security.SSLSetError, 0, objc._C_ID)
        self.assertArgHasType(Security.SSLSetError, 1, objc._C_INT)

    @expectedFailure
    @min_os_level("10.13")
    def test_functions10_13_missing(self):
        self.assertResultHasType(Security.SSLSetALPNProtocols, objc._C_INT)
        self.assertArgHasType(Security.SSLSetALPNProtocols, 0, objc._C_ID)
        self.assertArgHasType(Security.SSLSetALPNProtocols, 1, objc._C_ID)

        self.assertResultHasType(Security.SSLCopyALPNProtocols, objc._C_INT)
        self.assertArgHasType(Security.SSLCopyALPNProtocols, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SSLCopyALPNProtocols, 1, objc._C_OUT + objc._C_PTR + objc._C_ID
        )
        self.assertArgIsCFRetained(Security.SSLCopyALPNProtocols, 1)
